#pragma once

#include "FunctionInterfaces.h"
#include "BaseUserStudy.h"

template<typename T>
class AbstractFunction :
	public IFunction<T>,
	private _InputVal<T>,
	public CBaseUserStudy
{
protected:
	typedef	IFunction<T>						inherited;
	typedef	typename inherited::_Result			_Result;
	typedef	typename inherited::_Value			_Value;
protected:
	// return EL value of dataN, in ref param - real value
	inline	size_t						get_dataN_value(size_t& _pure){
		const size_t _dataN = ((NULL==m_DataN) ? m_const_DataN : m_DataN->Value());
		_pure=(IVariableFactory::_DEFAULT_DATA_N_==_dataN) ? *m_cur_dn_ref : _dataN-1;
		return _dataN;
	}

	inline	real_t						get_bb_value(){
		return (NULL==m_BarsBack) ? m_const_BarsBack : m_BarsBack->Value();
	}

	/************************************************************************/
	/* implementation of IFunctionBase										*/
	/************************************************************************/
public:
	_InputVal<size_t>*&		__cdecl		ctx_dataN(){return m_DataN;}
	_InputVal<real_t>*&		__cdecl		ctx_bb(){return m_BarsBack;}
	size_t&					__cdecl		ctx_dataN_const(){return m_const_DataN;}
	real_t&					__cdecl		ctx_bb_const(){return m_const_BarsBack;}
private:
	void*&					__cdecl		operator[](size_t _idx) {
		return CBaseUserStudyData::m_inputs.at(_idx)->access_ptr();
	}
	void					__cdecl		ReBindInputs(const bool _bind){
		if (!m_inputs.empty())
			m_ret.accept_influence_param(&m_inputs.front(),CBaseUserStudyData::m_inputs.size());
		CBaseUserStudyData::ReBindInputs(_bind);
	}
private:
							__cdecl		operator _InputVal<_Value>&(){return *this;}
							__cdecl		operator IInputsOwner&(){return *this;}
	void					__cdecl		DelayInitEnviroment(const size_t _dn) { m_ret.delay_init_enviroment(_dn); }
	void					__cdecl		SetCtxByDataNIsMyOrConst(const bool _is_const_or_my_dn) {
		m_ret.set_dn_type(_is_const_or_my_dn);
	}
/*	void					__cdecl		push_influence_input(const size_t _idx) OVERRIDE{
		m_ret.push_influence_param(CBaseUserStudyData::m_inputs.at(_idx)->get_input());
	}
*/	virtual _InputValInfo*			__cdecl		access_influence_for_input(const size_t _idx) OVERRIDE {
		return CBaseUserStudyData::m_inputs.at(_idx)->get_input();
	}
	virtual _InputValInfo*			__cdecl		access_bb() OVERRIDE {return m_BarsBack ? m_BarsBack : NULL ;}
	virtual _InputValInfo*			__cdecl		access_dn() OVERRIDE {return m_DataN ? m_DataN : NULL ;}
private:
	virtual bool					__cdecl		BarPersist()const OVERRIDE { return m_ret.is_master_bar_persist(); }
	virtual _InputValInfo::EBindedEntity
									__cdecl		GetEntityInfo()const OVERRIDE {return _InputValInfo::eBE_Variable;}
protected:
	explicit AbstractFunction(IBaseStudy* _master, _MRetVal<_Value>& _m_ret, size_t _flags)
		:	CBaseUserStudy(_master,_flags), m_ret(_m_ret),
			m_const_DataN(0), m_const_BarsBack(0),
			m_DataN(NULL), m_BarsBack(NULL), m_cur_dn_ref(_master->GetDataStream())
	{}
protected:
	size_t						m_const_DataN;
	real_t						m_const_BarsBack;
	_InputVal<size_t>*			m_DataN;
	_InputVal<real_t>*			m_BarsBack;
protected:
	_MRetVal<_Value>&			m_ret;
private:
	const size_t*const&			m_cur_dn_ref;
};

namespace MRET_MANAGE
{
	template<class TSimple> inline _MRetVal<TSimple>* FMRetCreator(const bool,IBaseStudy*const,IFunctionBase*const,void*const,const size_t,const TSimple*const);

	template<> inline _MRetVal<NumSimple>* FMRetCreator<NumSimple>(	const bool _is_series, 
																	IBaseStudy*const _p,
																	IFunctionBase*const _this,
																	void*const _parent,
																	const size_t _flags,
																	const NumSimple*const)
	{
		return _p->GetVarFactory().CreateNumeric_MRet(_is_series,_this,_parent,_flags);
	}

	template<> inline _MRetVal<BoolSimple>*	FMRetCreator<BoolSimple>(const bool _is_series,
																	IBaseStudy*const _p,
																	IFunctionBase*const _this,
																	void*const _parent,
																	const size_t _flags,
																	const BoolSimple*const)
	{
		return _p->GetVarFactory().CreateBool_MRet(_is_series,_this,_parent,_flags);
	}

	template<> inline _MRetVal<StringSimple>* FMRetCreator<StringSimple>(const bool _is_series,
																		IBaseStudy*const _p,
																		IFunctionBase*const _this,
																		void*const _parent,
																		const size_t _flags,
																		const StringSimple*const)
	{
		return _p->GetVarFactory().CreateString_MRet(_is_series,_this,_parent,_flags);
	}
};

#pragma warning (push)
#pragma warning (disable : 4355)

template<class TSimple, bool _ctx_disable_optim = false>
class SimpleFunction : public AbstractFunction<TSimple>
{
	typedef AbstractFunction<TSimple>						inherited;
	typedef	typename inherited::_Result						_Result;
	typedef	typename inherited::_Value						_Value;
	typedef SimpleFunction<TSimple,_ctx_disable_optim>		_MyT;
protected:
	inline	bool		SetCtx(real_t _bb) 
	{ 
		size_t _tmp_dataN(0);
		const real_t _bars_back=inherited::get_bb_value()+_bb;
		const size_t _dataN=inherited::get_dataN_value(_tmp_dataN);
		bool _need_set_ctx = _ctx_disable_optim || !(0==_bars_back && 0==_dataN);
		if (_need_set_ctx)
			inherited::m_Master->SetContextEx(false,this,_bars_back,_dataN);
		inherited::m_ret.i_am_running_now(_tmp_dataN);
		return _need_set_ctx;
	}
private:
	_Result	__cdecl		Value(real_t _bb){return _MyT::operator()(_bb);}
public:
	_Result	__cdecl		operator()(real_t _bb)
	{
		const bool _ctx_is_set=SetCtx(_bb);
		this->Body();
		if (_ctx_disable_optim || _ctx_is_set)
			inherited::m_Master->RestoreContextEx();
		return (m_ret_stubs.get()=inherited::m_ret.Value());
	}
protected:
	void	__cdecl		fake_run(size_t){}
protected:
	explicit SimpleFunction(IBaseStudy* _master, void* _parent, size_t _flags)
		:	inherited(_master, *(MRET_MANAGE::FMRetCreator(false,_master,this,_parent,_flags,(const TSimple*)NULL)), _flags)
	{}
private:
	// for simple functions it's needed!
	_dummy_ret_vals::_dummy_ret_vals<_Value,10>	m_ret_stubs;
};

template<class TSimple>
class SeriesFunction : public AbstractFunction<TSimple>
{
	typedef AbstractFunction<TSimple>	inherited;
	typedef	typename inherited::_Result	_Result;
	typedef	typename inherited::_Value	_Value;
	typedef SeriesFunction<TSimple>		_MyT;
protected:
	// return individual applied bars back value
	inline	bool		SetCtx(real_t _bb, real_t& _tmp_bb)
	{
		_tmp_bb=inherited::get_bb_value()+_bb;
		size_t _tmp_dataN;
		const size_t _dataN=inherited::get_dataN_value(_tmp_dataN);
		inherited::m_Master->SetContextEx(true,this,_tmp_bb,_dataN);
		return inherited::m_ret.i_am_running_now(_tmp_dataN);
	}

	inline	void		SetCtxForCheckRun(size_t _dataN)
	{
		// last param - pure data N value - real data stream number
		inherited::m_Master->SetContextEx(true,this,0,_dataN+1);
		inherited::m_ret.i_am_running_now(_dataN);
	}
private:
	_Result	__cdecl		Value(real_t _bb){return _MyT::operator()(_bb);}
public:
	// implementation of IFunction<>
	_Result	__cdecl		operator()(real_t _bb)
	{
		real_t _bb_self(0);
		const bool _need_run=SetCtx(_bb,_bb_self);
		if (_need_run)
			this->Body();
		inherited::m_Master->RestoreContextEx();
		return inherited::m_ret.Value(_bb_self);
	}
protected:
	// implementation of IFunctionBase 
	void	__cdecl		fake_run(size_t _dataN)
	{
		SetCtxForCheckRun(_dataN);
		this->Body();
		inherited::m_Master->RestoreContextEx();
	}
private:
	void	__cdecl		CloseBarVars(size_t _ds){inherited::m_ret.close_bar_func(_ds);}
	void	__cdecl		RecoveryVars(size_t _ds){inherited::m_ret.recovery_func(_ds);}
protected:
	explicit SeriesFunction(IBaseStudy* _master, void* _parent, size_t _flags)
		:	inherited(_master, *(MRET_MANAGE::FMRetCreator(true,_master,this,_parent,_flags,(const TSimple*)NULL)), _flags)
	{}
};

#pragma warning (pop)

typedef SimpleFunction<NumSimple>			NumericSimpleFunction;
typedef SimpleFunction<BoolSimple>			TrueFalseSimpleFunction;
typedef SimpleFunction<StringSimple>		StringSimpleFunction;

typedef SimpleFunction<NumSimple,true>		NumericSimpleFunction_CTX_OPTIM_DISABLED;
typedef SimpleFunction<BoolSimple,true>		TrueFalseSimpleFunction_CTX_OPTIM_DISABLED;
typedef SimpleFunction<StringSimple,true>	StringSimpleFunction_CTX_OPTIM_DISABLED;

typedef SeriesFunction<NumSimple>			NumericSeriesFunction;
typedef SeriesFunction<BoolSimple>			TrueFalseSeriesFunction;
typedef SeriesFunction<StringSimple>		StringSeriesFunction;
