#ifndef ELAPITYPES_H
#define ELAPITYPES_H

#include <algorithm>
#include <vector>

//#include "StudyTypes.h"

#include "StaticArray.h"

/*

inline bool CanGetPrevValue(GNType type)
{
	if(type==gnFunction){return true;}
	return false;
}
*/
enum E_StorageType
{
	EStorageError=-1,//               
	EStorageAuto=0,
	EStorageSimple=1,
	EStorageSeries=2,
	EStorageSeriesArg=3,
	EStorageProcedure=4
};

//////////////////////////////////////////////////////////////////////////
enum ECorrectlyStatus{
						eFullCorrectly=4,
						eValueInCorrectly=1,
						eVisualInCorrectly=2,
						eInCorrectly=3,
						eUnknownCorrectly=0
						};


class CELVarType;
class CELVarTypeEx;

class ELAPIType
{
	
public:
	//friend class boost::serialization::access;
	 template <class Archive>
	friend void serialize(Archive& ar,ELAPIType   & data,const unsigned int version);
	
	enum eFilter
	{
		eEasyTypeFilter		=0x1,
		eStorageTypeFilter	=0x10,
		eDimTypeFilter		=0x100,
		eRefTypeFilter		=0x1000
	};

	//////////////////////////////////////////////////////////////////////////
#include "EnumInDefFiles.h"
#include "ELTypeEnums.mh"
#include "UndefEnumInDefFiles.h"
	//////////////////////////////////////////////////////////////////////////



	//////////////////////////////////////////////////////////////////////////

	
	//////////////////////////////////////////////////////////////////////////
	typedef ELAPITypesHelpers::CStaticArray<int,10>		TELAPITypeDimsVector;
	TELAPITypeDimsVector		m_Dims;
	static const TELAPITypeDimsVector::value_type	_DYNAMIC_INDEX_=0;

private:	

	eStorageVarTypes	m_StorageType;
	eEasyTypes			m_EasyTypes;	
	eDimTypes			m_DimTypes;
	eRefTypes			m_RefTypes;
public:
	void Create(
				eEasyTypes			EasyTypes=eNumeric,
				eStorageVarTypes	StorageType=eSimple,	
				eDimTypes			DimTypes=eSingle,
				eRefTypes			RefTypes=eNotRef)
	{
		m_EasyTypes=EasyTypes;
		m_StorageType=StorageType;
		m_DimTypes=DimTypes;
		m_RefTypes=RefTypes;
	}
	///
	eStorageVarTypes	GetStorageType()	const {return m_StorageType;}
	eEasyTypes			GetEasyTypes()		const {return m_EasyTypes;}
	eDimTypes			GetDimTypes()		const {return m_DimTypes;}
	eRefTypes			GetRefTypes()		const {return m_RefTypes;}
	void SetStorageType(eStorageVarTypes v)	{m_StorageType = v;}
	void SetEasyTypes(eEasyTypes v)			{m_EasyTypes = v;}
	void SetDimTypes(eDimTypes v)			{m_DimTypes = v;}
	void SetRefTypes(eRefTypes v)			{m_RefTypes = v;}

	template<class Iter>
	void SetDims(Iter _begin, Iter _end)
	{
		m_Dims.assign(_begin,_end);
	}
	void SetAsDynamicArray(){ m_Dims.clear(); m_Dims.resize(1,_DYNAMIC_INDEX_); }

	const TELAPITypeDimsVector& GetDims()const
	{
		return m_Dims;
	}
	bool IsSimple()const ;
	bool IsSeries()const;
	bool IsUnknown()const;
	bool IsNotRef()const;
	bool IsRef()const;
	bool IsSingle()const;
	bool IsArray()const;
	bool IsDynamicArray()const;
	//ELAPIType(CELVarType ExtType);
	ELAPIType(CELVarType ExtType);
	void Create(CELVarType ExtType);
	ELAPIType(const CELVarTypeEx &ExtType);	
	ELAPIType()
	{
		Create();
	}
	ELAPIType(
		eEasyTypes			EasyTypes,
		eStorageVarTypes	StorageType,	
		eDimTypes			DimTypes,
		eRefTypes			RefTypes)
	{
		Create(EasyTypes,StorageType,DimTypes,RefTypes);
	}

 
	bool operator==(const ELAPIType &anoth)const
	{	return	anoth.m_Dims == m_Dims &&
	anoth.m_DimTypes == m_DimTypes &&
	anoth.m_EasyTypes == m_EasyTypes &&
	anoth.m_RefTypes == m_RefTypes &&
	anoth.m_StorageType	== m_StorageType;	}

	bool operator!=(const ELAPIType &anoth)const
	{	return !operator==(anoth);	}

	ELAPIType &operator=(const ELAPIType &anoth)
	{	m_Dims = anoth.m_Dims;
		m_DimTypes = anoth.m_DimTypes;
		m_EasyTypes = anoth.m_EasyTypes;
		m_RefTypes = anoth.m_RefTypes;
		m_StorageType = anoth.m_StorageType;
	return *this;	}

};

class CELVarType
{
public:

	enum ELVarType 
	{
	etError=-1,

	etNumericSimple =
		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eNotRef,		
	etNumeric =
		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eNotRef,						
	etNumericSimpleRef =		
		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eRef,						
	etNumericRef =
		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eRef,							
	etNumericSimplePointer =
		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::ePointer,
//	etNumericPointer =
//		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::ePointer,
	etNumericArraySimple =
		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etNumericArray =
		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etNumericArraySimpleRef =
		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eRef,							
	etNumericArrayRef =
		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eRef,							
//	etNumericArraySimplePointer =
//		ELAPIType::eNumeric | ELAPIType::eSimple | ELAPIType::eArray | ELAPIType::ePointer,
//	etNumericArrayPointer =
//		ELAPIType::eNumeric | ELAPIType::eSeries | ELAPIType::eArray | ELAPIType::ePointer,

		/*
	etlpNumeric =
		ELAPIType::elpNumeric | ELAPIType::eSimple | ELAPIType::eSingle  | ELAPIType::eNotRef,							
*/	
	etC_StrSimple = 
		ELAPIType::eC_Str  | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eNotRef,		
	etStringSimple =
		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eNotRef,		
	etString =
		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eNotRef,						
	etStringSimpleRef =		
		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eRef,						
	etStringRef =
		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eRef,							
	etStringSimplePointer =
		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::ePointer,
//	etStringPointer =
//		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::ePointer,
	etStringArraySimple =
		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etStringArray =
		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etStringArraySimpleRef =
		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eRef,							
	etStringArrayRef =
		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eRef,							
//	etStringArraySimplePointer =
//		ELAPIType::eString | ELAPIType::eSimple | ELAPIType::eArray | ELAPIType::ePointer,
//	etStringArrayPointer =
//		ELAPIType::eString | ELAPIType::eSeries | ELAPIType::eArray | ELAPIType::ePointer,

	etTrueFalseSimple =
		ELAPIType::eTrueFalse | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eNotRef,		
	etTrueFalse =
		ELAPIType::eTrueFalse | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eNotRef,						
	etTrueFalseSimpleRef =		
		ELAPIType::eTrueFalse | ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::eRef,						
	etTrueFalseRef =
		ELAPIType::eTrueFalse | ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::eRef,							
	etTrueFalseSimplePointer =
		ELAPIType::eTrueFalse| ELAPIType::eSimple | ELAPIType::eSingle | ELAPIType::ePointer,
//	etTrueFalsePointer =
//		ELAPIType::eTrueFalse| ELAPIType::eSeries | ELAPIType::eSingle | ELAPIType::ePointer,
	etTrueFalseArraySimple =
		ELAPIType::eTrueFalse | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etTrueFalseArray =
		ELAPIType::eTrueFalse | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eNotRef,							
	etTrueFalseArraySimpleRef =
		ELAPIType::eTrueFalse | ELAPIType::eSimple | ELAPIType::eArray  | ELAPIType::eRef,							
	etTrueFalseArrayRef =
		ELAPIType::eTrueFalse | ELAPIType::eSeries | ELAPIType::eArray  | ELAPIType::eRef,							
//	etTrueFalseArraySimplePointer =
//		ELAPIType::eTrueFalse| ELAPIType::eSimple | ELAPIType::eArray | ELAPIType::ePointer,
//	etTrueFalseArrayPointer =
//		ELAPIType::eTrueFalse| ELAPIType::eSeries | ELAPIType::eArray | ELAPIType::ePointer,

	};
	
	ELVarType Type ;
	CELVarType(ELVarType ExtType){Type=ExtType;}
	void Create(ELAPIType::eEasyTypes			EasyTypes=ELAPIType::eNumeric,
				ELAPIType::eStorageVarTypes		StorageType=ELAPIType::eSimple,	
				ELAPIType::eDimTypes			DimTypes=ELAPIType::eSingle,
				ELAPIType::eRefTypes			RefTypes=ELAPIType::eNotRef
				)
	{
		Type=CELVarType::ELVarType( EasyTypes|StorageType |DimTypes | RefTypes);
	}
	CELVarType(	ELAPIType::eEasyTypes			EasyTypes,
				ELAPIType::eStorageVarTypes		StorageType=ELAPIType::eSimple,	
				ELAPIType::eDimTypes			DimTypes=ELAPIType::eSingle,
				ELAPIType::eRefTypes			RefTypes=ELAPIType::eNotRef
				)
	{
		Create(EasyTypes,StorageType,DimTypes,RefTypes);
	}

	void Create(const ELAPIType &ExtType)
	{
		Type=CELVarType::ELVarType(ExtType.GetEasyTypes() | ExtType.GetStorageType() |ExtType.GetDimTypes() | ExtType.GetRefTypes());
	}
	CELVarType(const ELAPIType &ExtType)
	{
		Create(ExtType);
	}
	CELVarType()
	{
		ELAPIType ExtType;
		Create(ExtType);
	}
	ELAPIType::eStorageVarTypes		GetStorageType()	const {ELAPIType temp(*this); return temp.GetStorageType();}
	ELAPIType::eEasyTypes			GetEasyTypes()		const {ELAPIType temp(*this) ;return temp.GetEasyTypes();}
	ELAPIType::eDimTypes			GetDimTypes()		const {ELAPIType temp(*this); return temp.GetDimTypes();}
	ELAPIType::eRefTypes			GetRefTypes()		const {ELAPIType temp(*this); return temp.GetRefTypes();}



};


inline void ELAPIType::Create(CELVarType ExtType)
{
	Create(
		eEasyTypes(		eEasyTypeFilter		* 0xF  &  ExtType.Type),
		eStorageVarTypes(eStorageTypeFilter * 0xF  &  ExtType.Type),
		eDimTypes(		eDimTypeFilter		* 0xF  &  ExtType.Type),
		eRefTypes(		eRefTypeFilter		* 0xF  &  ExtType.Type)
		);

};
inline ELAPIType::ELAPIType(CELVarType ExtType)
{
Create(ExtType);
}


//////////////////////////////////////////////////////////////////////////
inline bool ELAPIType::IsSimple()const
{
	return (m_StorageType & ELAPIType::eStorageTypeFilter) == ELAPIType::eSimple;
}

inline bool ELAPIType::IsSeries()const
{
	return (m_StorageType & ELAPIType::eStorageTypeFilter) == ELAPIType::eSeries;
}

inline bool ELAPIType::IsUnknown()const
{
	return (m_StorageType & ELAPIType::eStorageTypeFilter) == ELAPIType::eUnknown;

};

inline bool ELAPIType::IsNotRef()const
{
	return (m_RefTypes & ELAPIType::eRefTypeFilter) == ELAPIType::eNotRef;
}

inline bool ELAPIType::IsRef()const
{
	return (m_RefTypes & ELAPIType::eRefTypeFilter) == ELAPIType::eRef;
}

inline bool ELAPIType::IsSingle()const
{
	return (m_DimTypes & ELAPIType::eDimTypeFilter) == ELAPIType::eSingle;
}

inline bool ELAPIType::IsArray()const
{
	return (m_DimTypes & ELAPIType::eDimTypeFilter) == ELAPIType::eArray;
}

inline bool ELAPIType::IsDynamicArray()const
{
	if (IsArray()) if (1==m_Dims.size()) return _DYNAMIC_INDEX_==m_Dims.at(0);
	return false;
}





inline CELVarType::ELVarType API2Var(const ELAPIType & apiType)
{
	return CELVarType::ELVarType(apiType.GetEasyTypes() | apiType.GetStorageType() |apiType.GetDimTypes() | apiType.GetRefTypes());
}


 
inline ELAPIType Var2API(const CELVarType::ELVarType & varType)
{
	ELAPIType apiType;
	apiType.Create(
		ELAPIType::eEasyTypes(		ELAPIType::eEasyTypeFilter	  *	0xF  &  varType),
		ELAPIType::eStorageVarTypes(ELAPIType::eStorageTypeFilter * 0xF  &  varType),
		ELAPIType::eDimTypes(		ELAPIType::eDimTypeFilter	  * 0xF  &  varType),
		ELAPIType::eRefTypes(		ELAPIType::eRefTypeFilter	  * 0xF  &  varType));
	return apiType;
}


inline CELVarType::ELVarType MakeRetValue(ELAPIType::eEasyTypes easyT, ELAPIType::eStorageVarTypes storT)
{
	return CELVarType::ELVarType(easyT | storT | ELAPIType::eSingle | ELAPIType::eNotRef);
}

inline 	void SetStorage(CELVarType::ELVarType &src,ELAPIType::eStorageVarTypes  stor)
	{
		int temp=src;
		temp|=ELAPIType::eStorageTypeFilter;//0x00f0;
		temp^=ELAPIType::eStorageTypeFilter;//0x00f0;
		temp|=stor;
		src=(CELVarType::ELVarType)temp;
	}
//////////////////////////////////////////////////////////////////////////
inline bool IsValidEasyType(ELAPIType::eEasyTypes easyT)
{
	return (easyT == ELAPIType::eNumeric) || (easyT == ELAPIType::eString) || (easyT == ELAPIType::eTrueFalse);
}

namespace ELStandard
{
#include "EnumInDefFiles.h"
	#include "ELSeries.mh"
#include "UndefEnumInDefFiles.h"
}

#include "EnumInDefFiles.h"
#include "EPrintTargetType.mh"
#include "UndefEnumInDefFiles.h"

#endif
