#pragma once

typedef double NumSimple;

#include "String_Light.h"
typedef  STLLight::_string	StringSimple;

typedef  NumSimple		NumericSimple;
typedef  NumSimple		real_t;
typedef  bool			BoolSimple;
typedef	 size_t			CountSimple;

typedef  NumericSimple	numericsimple;
typedef  NumSimple		numsimple;
typedef  BoolSimple		boolsimple;
typedef  StringSimple	stringsimple;

template<class T>	struct TELTypesIniter{};
template<>			struct TELTypesIniter<NumSimple>{ operator NumSimple(){return 0;} };
template<>			struct TELTypesIniter<size_t>{ operator size_t(){return 0;} };
template<>			struct TELTypesIniter<BoolSimple>{ operator BoolSimple(){return false;} };
template<>			struct TELTypesIniter<StringSimple>
{
	operator const char*(){return "";}
	operator StringSimple(){return "";}
};

namespace _ELAPI_Numeric_Compare
{
	template<class _Arg, class _Result>
	struct base_compare_function
	{
		typedef _Arg	first_argument_type;
		typedef _Arg	second_argument_type;
		typedef _Result	result_type;

		explicit base_compare_function(const _Arg& _epsilon) : m_epsilon(_epsilon){}
	protected:
		static inline _Arg ___abs_dif__(const _Arg& lval, const _Arg& rval) { return ((lval>rval)?lval-rval:rval-lval); }
	protected:
		const _Arg	m_epsilon;
	};

	template<class _Ty>
	struct less :
		public base_compare_function<_Ty,bool>
	{
		typedef base_compare_function<_Ty,bool>	inherited;

		explicit less(const _Ty& _epsilon) : inherited(_epsilon) {}

		bool operator()(const _Ty& _lval, const _Ty& _rval) const
		{
			return _lval < _rval && inherited::___abs_dif__(_lval,_rval) > inherited::m_epsilon;
		}
	};

	template<class _Ty>
	struct greate :
		public base_compare_function<_Ty,bool>
	{
		typedef base_compare_function<_Ty,bool>	inherited;

		explicit greate(const _Ty& _epsilon) : inherited(_epsilon) {}

		bool operator()(const _Ty& _lval, const _Ty& _rval) const
		{
			return _lval > _rval && inherited::___abs_dif__(_lval,_rval) > inherited::m_epsilon;
		}
	};

	template<class _Ty>
	struct equal :
		public base_compare_function<_Ty,bool>
	{
		typedef base_compare_function<_Ty,bool>	inherited;

		explicit equal(const _Ty& _epsilon) : inherited(_epsilon) {}

		bool operator()(const _Ty& _lval, const _Ty& _rval) const
		{
			return inherited::___abs_dif__(_lval,_rval) <= inherited::m_epsilon;
		}
	};

	template<class _Ty>
	struct less_equal :
		public base_compare_function<_Ty,bool>
	{
		typedef base_compare_function<_Ty,bool>	inherited;

		explicit less_equal(const _Ty& _epsilon) : inherited(_epsilon) {}

		bool operator()(const _Ty& _lval, const _Ty& _rval) const
		{
			return less<_Ty>(inherited::m_epsilon)(_lval,_rval) || equal<_Ty>(inherited::m_epsilon)(_lval,_rval);
		}
	};

	template<class _Ty>
	struct greate_equal :
		public base_compare_function<_Ty,bool>
	{
		typedef base_compare_function<_Ty,bool>	inherited;

		explicit greate_equal(const _Ty& _epsilon) : inherited(_epsilon) {}

		bool operator()(const _Ty& _lval, const _Ty& _rval) const
		{
			return greate<_Ty>(inherited::m_epsilon)(_lval,_rval) || equal<_Ty>(inherited::m_epsilon)(_lval,_rval);
		}
	};
};

#define __NumSimple__epsilon__	+1e-12

inline bool NumSimpleGreate	(const NumSimple _lval, const NumSimple _rval)
{
	return _ELAPI_Numeric_Compare::greate<NumericSimple>(__NumSimple__epsilon__)(_lval,_rval);
}

inline bool NumSimpleLess	(const NumSimple _lval, const NumSimple _rval)
{
	return _ELAPI_Numeric_Compare::less<NumericSimple>(__NumSimple__epsilon__)(_lval,_rval);
}

inline bool NumSimpleEqual	(const NumSimple _lval, const NumSimple _rval)
{
	return _ELAPI_Numeric_Compare::equal<NumericSimple>(__NumSimple__epsilon__)(_lval,_rval);
}

#undef __NumSimple__epsilon__

inline bool NumSimpleNEqual	(const NumSimple _lval, const NumSimple _rval) { return !NumSimpleEqual(_lval,_rval); }
inline bool NumSimpleLEqual	(const NumSimple _lval, const NumSimple _rval) { return NumSimpleLess(_lval,_rval) || NumSimpleEqual(_lval,_rval); }
inline bool NumSimpleGEqual	(const NumSimple _lval, const NumSimple _rval) { return NumSimpleGreate(_lval,_rval) || NumSimpleEqual(_lval,_rval); }

struct IExceptionGenerate
{
	struct IExceptionSaver
	{
		virtual void	__stdcall reset(const void*const)=0;
	};

	virtual void				__stdcall _throw(const int=-1, const char* ="")const=0;
	virtual void				__stdcall _throw_ex(const char*)const=0;
	virtual IExceptionSaver&	__stdcall get_setter()const=0;
};