#pragma once

#include "VariableInterfaces.h"
#include "PrintInfo.h"
#include "ELRW_IImpl.h"
#include "PriceSeries.h"

#if defined(_MSC_VER) || defined(_BCB_VER)
#include "ELAPITypes.h"
#endif

#include "TWS_OrdersInterfaces.h"

struct	IEasyLanguageObject;
struct	IExpIndicatorDebugger;
class	IDllToolKit;
struct	IFunctionBase;
struct	IVariableContainerRead;
struct	IGlobalSeriesProxyFactory;

enum EVariableAttributes
{
	eVASimple					=	0x00000001,
	eVASeries					=	0x00000002,

	eVAOrdinal					=	0x00000010,
	eVAIntrabarPersist			=	0x00000020,

	eVA_Fixed_Data_Stream		=	0x00000100,
	eVA_Variable_Data_Stream	=	0x00000200,

	eVA_Hidden					=	0x00001000, //not available through PLKit API.
};

enum EFunctionAttributes
{
	eFA_OptimisationDenied		=	0x00000001,
	eFA_AllowOptimisation		=	0x00000002,

	eFA_Fixed_Data_Stream		=	eVA_Fixed_Data_Stream,
	eFA_Variable_Data_Stream	=	eVA_Variable_Data_Stream,
};

struct IUserStudiesAllocator
{
	virtual	void*		__cdecl	allocate(const size_t _size)const=0;
	virtual	void		__cdecl	deallocate(void*)const=0;
};

namespace _ELAPI_{ namespace variables{
	struct IVariableManage;
}}

struct IVariableFactory
{
	typedef NumericSimple	(__cdecl * TPFNum)();
	typedef size_t			(__cdecl * TPFUint)();
	typedef BoolSimple		(__cdecl * TPFBool)();
	typedef const char*		(__cdecl * TPFStr)();

	static const size_t _DEFAULT_DATA_N_=0;
	// for creating variables
	// first param : true - series, false - simple;
	// second param : master of variable - if NULL then master is Indicator
	virtual	IVariable<NumSimple>*				__cdecl	CreateNumeric(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;
	virtual	IVariable<BoolSimple>*				__cdecl	CreateBool(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;
	virtual	IVariable<StringSimple>*			__cdecl	CreateString(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;

	virtual	IVariable<IArray<NumSimple> >*		__cdecl	CreateNumericArr(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;
	virtual	IVariable<IArray<BoolSimple> >*		__cdecl	CreateBoolArr(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;
	virtual	IVariable<IArray<StringSimple> >*	__cdecl	CreateStringArr(size_t,const EVariableAttributes,IFunctionBase*,const char*,size_t)=0;

	virtual	void								__cdecl	RegisterNumericVar(_VariableManageBase*const,size_t,const EVariableAttributes,IFunctionBase*,const char*)=0;
	virtual	void								__cdecl	RegisterBoolVar(_VariableManageBase*const,size_t,const EVariableAttributes,IFunctionBase*,const char*)=0;
	virtual	void								__cdecl	RegisterStringVar(_VariableManageBase*const,size_t,const EVariableAttributes,IFunctionBase*,const char*)=0;

	// el_jit will be call this methods for creating function inputs 
	virtual	_InputVal<NumSimple>*				__cdecl	CreateNumericInpEx(TPFNum)=0;
	virtual	_InputVal<BoolSimple>*				__cdecl	CreateBoolInpEx(TPFBool)=0;
	virtual	_InputVal<StringSimple>*			__cdecl	CreateStringInpEx(TPFStr)=0;

	virtual	_InputVal<size_t>*					__cdecl	CreateDataNInpEx(TPFUint)=0;
	virtual	_InputVal<real_t>*					__cdecl	CreateBarsBackInpEx(TPFNum)=0;

// void* param is parent of function - for hierarchical grouping
	//for creating m_rets of functions - directly not used, used implicitly in ctor of functions
	virtual	_MRetVal<NumSimple>*				__cdecl	CreateNumeric_MRet(bool,IFunctionBase*,void*,const size_t)=0;
	virtual	_MRetVal<BoolSimple>*				__cdecl	CreateBool_MRet(bool,IFunctionBase*,void*,const size_t)=0;
	virtual	_MRetVal<StringSimple>*				__cdecl	CreateString_MRet(bool,IFunctionBase*,void*,const size_t)=0;

	virtual	void								__cdecl	_RegisterFunction(
		const bool _is_series, IFunctionBase*const _func,
		_ELAPI_::variables::IVariableManage*const _m_ret, void*const _parent
	)=0;

	// for f(g(h())) call's
	virtual	void								__cdecl	SetParent(IFunctionBase*,IFunctionBase*)=0;

	// for enabling/disabling optimization
	virtual	void								__cdecl	SetOptimisationAttributes(IFunctionBase*,const EFunctionAttributes)=0;

	virtual const IUserStudiesAllocator&		__cdecl GetUserStudiesAllocator()const=0;

	virtual	_InputVal<NumSimple>*				__cdecl CreateNumericInputVar(const char* name)=0;
	virtual	_InputVal<BoolSimple>*				__cdecl CreateTrueFalseInputVar(const char* name)=0;
	virtual	_InputVal<StringSimple>*			__cdecl CreateStringInputVar(const char* name)=0;
};

template<class T>
class IDispetcherStub
{
public:
  virtual T* __stdcall GetStub()=0;
};

struct SPlotDeclareInfo;
class SOrderWordInfoImpl;

struct IPlot
{
	virtual	void					__cdecl				SetPlotValue(size_t,real_t,NumSimple,const char* =NULL,int=Default,int=Default,int=Default)=0;
	virtual	void					__cdecl				SetPlotValueS(size_t,real_t,const char*,const char* =NULL,int=Default,int=Default,int=Default)=0;
	virtual	NumSimple&				__cdecl				GetPlotValue(size_t,real_t)=0;
	
	virtual void					__cdecl				noplot(size_t, real_t = 0.0) = 0;
	virtual	void					__cdecl				setplottype(size_t number, int type)=0;
	virtual void					__cdecl				setplotbgcolor(real_t bb, size_t number, int color)=0;
	virtual void					__cdecl				setplotcolor(real_t bb, size_t number, int color)=0;
	virtual void					__cdecl				setplotwidth(real_t bb, size_t number, int width)=0;

	virtual int						__cdecl				sl_getplotcolor(size_t number, real_t bb)=0;
	virtual int						__cdecl				sl_getplotwidth(size_t number, real_t bb)=0;
	virtual int						__cdecl				sl_getplotBGcolor(size_t number, real_t bb)=0;
	
	virtual void					__cdecl				plotpaintbar4(real_t,NumSimple,NumSimple,NumSimple,NumSimple,const char* =NULL, int=Default, int=Default, int=Default) = 0;
	virtual void					__cdecl				plotpaintbar2(real_t,NumSimple,NumSimple,const char* =NULL,int=Default,int=Default,int=Default) = 0;

	virtual	size_t					__cdecl				AddPlot(const SPlotDeclareInfo& _info)=0;
};

//	strategy orders
struct IStrategyOrders
{
	virtual void					__cdecl				SaveOrder(size_t,int=0)=0;
	virtual void					__cdecl				SavePricedOrder(size_t,double,int=0)=0;
	virtual void					__cdecl				SaveOrder2(size_t, double, double, int=0)=0;
	virtual void					__cdecl				setbreakeven(NumSimple)=0;
	virtual void					__cdecl				setdollartrailing(NumSimple)=0;
	virtual void					__cdecl				setexitonclose()=0;
	virtual void					__cdecl				setpercenttrailing(NumSimple,NumSimple)=0;
	virtual void					__cdecl				setprofittarget(NumSimple)=0;
	virtual void					__cdecl				setstoploss(NumSimple)=0;
	virtual void					__cdecl				setstopshare()=0;
	virtual void					__cdecl				setstopcontract()=0;
	virtual void					__cdecl				setstopposition()=0;

	virtual void					__cdecl				SetEntriesPriority(_InputValBase<NumSimple>*const)=0;

	virtual NumSimple				__cdecl				portfolio_calcmaxpotentiallossforentry(int _side, int _contracts=0, NumSimple _price=0)=0;
	virtual bool					__cdecl				portfolio_setmaxpotentiallosspercontract(NumSimple _val)=0;

	virtual bool					__cdecl				PlaceMarketOrder(bool _is_buy, bool _is_entry, int _cntrcts=0)=0;
	virtual void					__cdecl				ChangeMarketPosition(real_t _delta, real_t _price, const char* _name=0 )=0;

	virtual size_t					__cdecl				AddOrder(const SOrderWordInfoImpl& _info)=0;

	virtual NumSimple				__cdecl				actentry_tradecount()=0;
	virtual NumSimple				__cdecl				actentry_date(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_time(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_price(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_exitdate(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_exittime(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_exitprice(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_shares(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_profit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_maxprofit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_minprofit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_contractprofit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_maxcontractprofit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_mincontractprofit(real_t _entry_index=0)=0;
	virtual NumSimple				__cdecl				actentry_comissionvalue(real_t _entry_index=0)=0;
	virtual real_t					__cdecl				position_trades_count(real_t _pos_num)=0;
	virtual real_t					__cdecl				pos_trade_size(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_comission(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_profit(real_t _pos_num, real_t _trade_num)=0;

	virtual const char*				__cdecl				pos_trade_Ename(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Eprice(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Ebar(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Ecategory(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Edatetime(real_t _pos_num, real_t _trade_num)=0;

	virtual const char*				__cdecl				pos_trade_Xname(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Xprice(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Xbar(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Xcategory(real_t _pos_num, real_t _trade_num)=0;
	virtual real_t					__cdecl				pos_trade_Xdatetime(real_t _pos_num, real_t _trade_num)=0;

	virtual bool					__cdecl				pos_trade_isLong(real_t _pos_num, real_t _trade_num)=0;
	virtual bool					__cdecl				pos_trade_isOpen(real_t _pos_num, real_t _trade_num)=0;

	virtual void					__cdecl				SetInitialBrokerPosition(int _market_pos, const char* _entry_name, real_t _avg_price, real_t _max_op)=0;
};

enum EInsideOrderCategory {
	eInsideStop = 1,
	eInsideLimit,
	eInsideMarket,
	eInsideClose,
	eInsideOpen,
	eInsideMarketRangeO,
	eInsideMarketRangeC
};

struct IIntrinsicsAccessor
{
	virtual intrinsic_functions::IntrinsicFunctions<NumSimple>&		__cdecl	GetIntrinsics(_InputVal<NumSimple>&)=0;
	virtual intrinsic_functions::IntrinsicFunctions<BoolSimple>&	__cdecl	GetIntrinsics(_InputVal<BoolSimple>&)=0;
};

class IBaseStudy
{
public:
    virtual IGlobalVar& 			__cdecl				open(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				high(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				low(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				close(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				volume(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				openint(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				date(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				time(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				ticks(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				upticks(size_t n=0) = 0;
    virtual IGlobalVar& 			__cdecl				downticks(size_t n=0) = 0;
	virtual IGlobalVar& 			__cdecl				time_s(size_t n=0) = 0;
	virtual IGlobalVar&				__cdecl				g_seria(const eELPriceSeriesMD _type, const size_t _n) = 0;//INSIDEn

	virtual IStdFunctions*			__stdcall 			GetPoolStandartFunctions()=0;//!0,0;INSIDEn
	virtual IPlot*					__stdcall 			GetPlots() = 0;//!0,0;INSIDEn
	virtual IStrategyOrders*		__stdcall 			GetStrategyOrdersGenerator()=0;//!0,0;INSIDEn
	virtual void*					__stdcall 			CreateFuncObj(const char*,void*,size_t=eFA_Fixed_Data_Stream)=0;//!0,0;INSIDEn
	virtual IDllToolKit*			__stdcall 			GetDllToolKit()=0;//!0,0;INSIDEn

	virtual IExpIndicatorDebugger*	__stdcall 			AccessDebugger()=0;//!0,0;INSIDEn

	virtual IEasyLanguageObject*	__stdcall 			SELF()=0;//!0,0;INSIDEn
	virtual const IVariableContainerRead*	__stdcall	GetContainer(size_t)const=0;//!0,0;INSIDEn

	virtual const size_t*const&		__stdcall			GetDataStream()=0;//!0,0;INSIDEn
	virtual	size_t					__stdcall			GetMaxDataStreamNumber()=0;//!0,0;INSIDEn
	virtual	bool					__stdcall			IsDataStreamExist(size_t DataN)=0;//!0,0;INSIDEn
	virtual	size_t					__stdcall			MaxAvaliableDataStreamNumber()=0;//!0,0;INSIDEn

	virtual void					__cdecl				alert(const char* _val=0)=0;
	virtual void					__cdecl				setalertstate(BoolSimple _val)=0;
	virtual BoolSimple				__cdecl				checkalert()const=0;
	virtual BoolSimple				__cdecl				alertenabled()const=0;
	virtual NumSimple				__cdecl				bigpointvalue(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				barstatus(size_t dataNum=0)=0;//!0,0;
	virtual NumSimple				__cdecl				commoditynumber()=0;//!0,0;w
	virtual const char*				__cdecl				getexchangename(size_t dataNum=0)=0;//!0,0;d
	virtual const char*				__cdecl				getsymbolname(size_t dataNum=0)=0;//!0,0;d
	virtual const char*				__cdecl				symbolroot(size_t dataNum=0)=0;//!0,0;d
	virtual const char*				__cdecl				Description(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				maxbarsforward()=0;
	virtual NumSimple				__cdecl				pointvalue(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				deliverymonth(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				deliveryyear(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				bartype(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				intervaltype(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				intervaltype_ex(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				expirationdate(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sess1starttime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sess1endtime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sess2starttime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sess2endtime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				unionsess1starttime()=0;//!0,0;
	virtual NumSimple				__cdecl				unionsess1endtime()=0;//!0,0;
	virtual NumSimple				__cdecl				unionsess2starttime()=0;//!0,0;
	virtual NumSimple				__cdecl				unionsess2endtime()=0;//!0,0;
	virtual NumSimple				__cdecl				sess1firstbartime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sess2firstbartime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				sessionstarttime(int sesstype, size_t sessnum, size_t dataNum=0)=0;//!2,0;d
	virtual NumSimple				__cdecl				sessionendtime(int sesstype, size_t sessnum, size_t dataNum=0)=0;//!2,0;d
	virtual NumSimple				__cdecl				sessioncount(int sesstype, size_t dataNum=0)=0;//!1,0;d
	virtual NumSimple				__cdecl				sessionstartday(int sesstype, size_t sessnum, size_t dataNum=0)=0;//!2,0;d
	virtual NumSimple				__cdecl				sessionendday(int sesstype, size_t sessnum, size_t dataNum=0)=0;//!2,0;d
	virtual NumSimple				__cdecl				sessionstarttimems(size_t sessnum)=0;
	virtual NumSimple				__cdecl				sessionendtimems(size_t sessnum)=0;
	virtual NumSimple				__cdecl				sessioncountms()=0;
	virtual NumSimple				__cdecl				sessionstartdayms(size_t sessnum)=0;
	virtual NumSimple				__cdecl				sessionenddayms(size_t sessnum)=0;

//////////////////////////////////////////////////////////////////////////
	virtual NumSimple				__cdecl				i_marketposition()=0;
	virtual NumSimple				__cdecl				i_avgentryprice()=0;
	virtual NumSimple				__cdecl				i_currentcontracts()=0;
	virtual NumSimple				__cdecl				i_currentshares()=0;
	virtual NumSimple				__cdecl				i_openequity()=0;
	virtual NumSimple				__cdecl				i_closedequity()=0;
	virtual NumSimple				__cdecl				i_marketposition_at_broker_for_the_strategy()=0;
	virtual NumSimple				__cdecl				i_marketposition_at_broker()=0;

	virtual NumSimple				__cdecl				percentprofit()=0; // { return numwintrades() - totaltrades(); }
	virtual NumSimple				__cdecl				avgbarseventrade()=0;
	virtual NumSimple				__cdecl				avgbarslostrade()=0;
	virtual NumSimple				__cdecl				avgbarswintrade()=0;
	virtual NumSimple				__cdecl				avgentryprice()=0;
	virtual NumSimple				__cdecl				grossloss()=0;
	virtual NumSimple				__cdecl				grossprofit()=0;
	virtual NumSimple				__cdecl				largestlostrade()=0;
	virtual NumSimple				__cdecl				largestwintrade()=0;
	virtual NumSimple				__cdecl				maxconseclosers()=0;
	virtual NumSimple				__cdecl				maxconsecwinners()=0;
	virtual NumSimple				__cdecl				maxcontractsheld()=0;
	virtual NumSimple				__cdecl				maxiddrawdown()=0;
	virtual NumSimple				__cdecl				maxsharesheld()=0;
	virtual NumSimple				__cdecl				numeventrades()=0;
	virtual NumSimple				__cdecl				numlostrades()=0;
	virtual NumSimple				__cdecl				numwintrades()=0;
	virtual NumSimple				__cdecl				totalbarseventrades()=0;
	virtual NumSimple				__cdecl				totalbarslostrades()=0;
	virtual NumSimple				__cdecl				totalbarswintrades()=0;
	virtual NumSimple				__cdecl				totalbarslostrade()=0;
	virtual NumSimple				__cdecl				totalbarswintrade()=0;
	virtual NumSimple				__cdecl				livedate()=0;
	virtual NumSimple				__cdecl				livetime()=0;
	virtual NumSimple				__cdecl				totaltrades()=0;

//////////////////////////////////////////////////////////////////////////

//	strategy profit information

	virtual NumSimple				__cdecl				marketposition(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				marketposition_at_broker_for_the_strategy()=0;
	virtual NumSimple				__cdecl				marketposition_at_broker()=0;
	
	virtual NumSimple				__cdecl				maxpositionprofit(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxpositionloss(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				openpositionprofit()=0;
	virtual NumSimple				__cdecl				positionprofit(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				contractprofit(real_t _pos=0)=0;//!1,0;

	virtual int						__cdecl				barssinceentry(real_t _pos=0)=0;//!1,0;
	virtual int						__cdecl				barssinceexit(real_t _pos=0)=0;//!1,0;
	
	virtual const char*				__cdecl				entryname(real_t _pos=0)=0;
	virtual	NumSimple				__cdecl				entrydate(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				entrytime(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				entryprice(real_t _pos=0)=0;//!1,0;
	virtual const char*				__cdecl				exitname(real_t _pos=0)=0;
	virtual NumSimple				__cdecl				exitdate(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				exittime(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				exitprice(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				currentcontracts()=0;
	virtual NumSimple				__cdecl				currentshares()=0;
	virtual NumSimple				__cdecl				currententries()=0;

	virtual NumSimple				__cdecl				maxcontractprofit(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxcontracts(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxentries(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxshares(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				netprofit()=0;

//////////////////////////////////////////////////////////////////////////
//
//	Checked function

	virtual NumSimple				__cdecl				maxpositionsago()=0;

	virtual NumSimple				__cdecl				marketposition_checked(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				maxpositionprofit_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxpositionloss_checked(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				positionprofit_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				contractprofit_checked(real_t _pos=0)=0;//!1,0;

	virtual int						__cdecl				barssinceentry_checked(real_t _pos=0)=0;//!1,0;
	virtual int						__cdecl				barssinceexit_checked(real_t _pos=0)=0;//!1,0;

	virtual	NumSimple				__cdecl				entrydate_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				entrytime_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				entryprice_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				exitdate_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				exittime_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				exitprice_checked(real_t _pos=0)=0;//!1,0;

	virtual NumSimple				__cdecl				maxcontractprofit_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxcontracts_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxentries_checked(real_t _pos=0)=0;//!1,0;
	virtual NumSimple				__cdecl				maxshares_checked(real_t _pos=0)=0;//!1,0;

//////////////////////////////////////////////////////////////////////////
	virtual NumSimple				__cdecl				portfolio_numlosstrades()=0;
	virtual NumSimple				__cdecl				portfolio_numwintrades()=0;
	virtual NumSimple				__cdecl				portfolio_percentprofit()=0;
	virtual NumSimple				__cdecl				portfolio_netprofit()=0;
	virtual NumSimple				__cdecl				portfolio_grossloss()=0;
	virtual NumSimple				__cdecl				portfolio_grossprofit()=0;
	virtual NumSimple				__cdecl				portfolio_totaltrades()=0;
	virtual NumSimple				__cdecl				portfolio_openpositionprofit()=0;
	virtual NumSimple				__cdecl				portfolio_currententries()=0;
	virtual NumSimple				__cdecl				portfolio_maxiddrawdown()=0;
	virtual NumSimple				__cdecl				portfolio_strategydrawdown()=0;
	virtual NumSimple				__cdecl				portfolio_maxriskequityperpospercent()=0;
	virtual NumSimple				__cdecl				portfolio_totalmaxriskequitypercent()=0;

	virtual NumSimple				__cdecl				portfolio_maxopenpositionpotentialloss()=0;
	virtual NumSimple				__cdecl				portfolio_getmaxpotentiallosspercontract()=0;
	virtual NumSimple				__cdecl				portfolio_getmarginpercontract()=0;

	virtual NumSimple				__cdecl				portfolio_investedcapital()=0;
	
//////////////////////////////////////////////////////////////////////////

	virtual NumSimple				__cdecl				commission()=0;
	virtual const char*				__cdecl				getstrategyname()=0;
	virtual NumSimple				__cdecl				margin(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				slippage()=0;
	virtual const char*				__cdecl				getaccount()=0;

//////////////////////////////////////////////////////////////////////////

	virtual const char*				__cdecl				getstudyname()=0;

	virtual NumSimple				__cdecl				insideask(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				insidebid(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				last(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				prevclose(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				tradevolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyvolumeup(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyvolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyvolumedn(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyvolumeuc(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailytradesup(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailytradesdn(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyhigh(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailylow(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyopen(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailyclose(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				currentopenint(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				tradedate(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				tradetime(size_t _dataN=0)=0;//!0,0;d
	virtual long					__cdecl				Category(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				ivolatility(real_t bb, size_t _dataN=0)=0;//!0,0;bd
	virtual NumSimple				__cdecl				putvolume(real_t bb, size_t _dataN=0)=0;//!0,0;bd
	virtual NumSimple				__cdecl				callvolume(real_t bb, size_t _dataN=0)=0;//!0,0;bd
	virtual NumSimple				__cdecl				putopenint(real_t bb, size_t _dataN=0)=0;//!0,0;bd
	virtual NumSimple				__cdecl				callopenint(real_t bb, size_t _dataN=0)=0;//!0,0;bd
	virtual NumSimple				__cdecl				high52wk(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				low52wk(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				GetDateTime(int barsBack=0)=0;//!0,0;b
	virtual NumSimple				__cdecl				lastcalcjdate(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				lastcalcmmtime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				lastcalcsstime(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				BarInterval(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				boxsize(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				revsize(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				DataCompression(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				getbackgroundcolor()=0;//!0,0;
	virtual size_t					__cdecl				currentbar(size_t n=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				pricescale(size_t n=0)=0;//!0,0;d
	virtual size_t					__cdecl				MaxBarsBack()=0;
	virtual size_t					__cdecl				GetMaxBarReference()=0;
	virtual NumSimple				__cdecl				minmove(size_t n=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				dailylimit(size_t n=0)=0;//!0,0;d
	virtual void					__cdecl				PrintToFile(const char* file, const iprint_func* format, va_list& marker)=0;//!0,0;INSIDEn
	virtual const char*				__cdecl				PrintToDest(EPrintTargetType destination, const iprint_func* format, va_list& marker)=0;//!0,0;INSIDEn
	virtual IExceptionGenerate*		__cdecl				_get_exception_generator()=0;//!0,0;INSIDEn
	virtual NumSimple				__cdecl				GetAppInfo(size_t _id)=0;
	virtual NumSimple				__cdecl				point(size_t _dataN=0)=0;//!0,0;d
	virtual const char*				__cdecl				Symbolname(size_t dataNum=0)=0;//!0,0;d
	virtual const char*				__cdecl				Symbol(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_bigpointvalue(size_t dataNum=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_ask(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_bid(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_last(size_t _dataN=0)=0;//!0,0;d

	virtual NumSimple				__cdecl				q_upvolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_totalvolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_downvolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_unchangedvolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_openinterest(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_tradevolume(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_date(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_time(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_time_s(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_callopenint(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl				q_previousclose(size_t _dataN=0)=0;//!0,0;d

//////////////////////////////////////////////////////////////////////////
/*
	virtual const char*				__cdecl 			q_askexchange(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_asksize(size_t _dataN=0)=0;//!0,0;d
	virtual const char*				__cdecl 			q_bidexchange(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_bidsize(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_expirationdate(size_t _dataN=0)=0;//!0,0;d

	virtual NumSimple				__cdecl 			q_lasttradingdate(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_margin(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			optiontype(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_optiontype(size_t _dataN=0)=0;//!0,0;d
	virtual NumSimple				__cdecl 			q_ticks(size_t _dataN=0)=0;//!0,0;d
*/
//////////////////////////////////////////////////////////////////////////

	virtual const char*				__cdecl				exchlisted(size_t _dataN=0)=0;//!0,0;d
	virtual const char*				__cdecl				q_exchangelisted(size_t _dataN=0)=0;//!0,0;d

	virtual void					__cdecl				raiseruntimeerror(const char* reason)=0;//!0,1;n
	
	virtual void					__cdecl				SetContext(bool _is_series, real_t _bb=0, size_t _dataN=0)=0;//!0,0;INSIDEn
	virtual void					__cdecl				RestoreContext()=0;//!0,0;INSIDEn

	virtual void					__cdecl				SetContextSimple(real_t _bb)=0;//INSIDEn
	virtual void					__cdecl				RestoreContextSimple()=0;//INSIDEn

	virtual IVariableFactory&		__cdecl				GetVarFactory()=0;//!0,0;INSIDEn
	virtual	NumSimple				__cdecl				ExecOffset()=0;//!0,0;INSIDEn
	virtual void					__cdecl				SetContextEx(bool _is_series, IFunctionBase* _pSetter, real_t _bb=0, size_t _dataN=0)=0;//!0,0;INSIDEn
	virtual void					__cdecl				RestoreContextEx()=0;//!0,0;INSIDEn

	virtual NumSimple				__cdecl				getplotbgcolor(size_t number, real_t bb=0)=0;//!0,0;b
	virtual NumSimple				__cdecl				getplotcolor(size_t number, real_t bb=0)=0;//!0,0;b
	virtual NumSimple				__cdecl				getplotwidth(size_t number, real_t bb=0)=0;//!0,0;b

	virtual NumSimple				__cdecl				bartype_ex(size_t dataNum=0)=0;//!0,0;d

	virtual BoolSimple				__cdecl				sessionlastbar(size_t dataNum=0)=0;//!0,0;d

	virtual size_t					__cdecl				basedatanumber()=0;
	virtual size_t					__cdecl				currentdatanumber()=0;

	virtual void					__cdecl				recalculate()=0;

	virtual ITWSOrdersManagment&	__cdecl				GetTWSOrderProxy()=0;

	virtual IGlobalSeriesProxyFactory&		__cdecl		GetGSeriesFactory()=0;
	
	virtual IIntrinsicsAccessor&	__cdecl				GetIntrinsicAccessor()=0;

	virtual NumSimple				__cdecl				vestedcapital()=0;

	virtual NumSimple				__cdecl				InitialCapital()const=0;

	virtual void					__cdecl				RecalcAfter( real_t seconds )=0;
};

class IUserBaseStudy :
	public IUserStudyVarHost
{
public:
	virtual	void	__cdecl		Construct(){}
	virtual	void	__cdecl		BindInputs()=0;
	virtual	void	__cdecl		ReBindInputs()=0;
	virtual	void	__cdecl		Initialize()=0;
	virtual	void	__cdecl		Body()=0;
	virtual	bool	__cdecl		LegacyColorValue()const=0;
};

struct E_TEXT{};

class CPrint_Wrap
{
public:
	explicit CPrint_Wrap(IBaseStudy* _ptr)
		:m_master(*_ptr){}
public:
	void			__cdecl operator()(const char* file, const iprint_func* format, ...)
	{
		va_list marker;
		va_start(marker, format);
		m_master.PrintToFile(file,format,marker);
	}
	void			__cdecl	operator()(EPrintTargetType destination, const iprint_func* format, ...)
	{
		va_list marker;
		va_start(marker, format);
		m_master.PrintToDest(destination,format,marker);
	}
	StringSimple	__cdecl operator()(E_TEXT, const iprint_func* format, ...)
	{
		va_list marker;
		va_start(marker, format);
		return m_master.PrintToDest(EText,format,marker);
	}
private:
	IBaseStudy&  m_master;
};


struct IExpIndicatorDebugger
{
	virtual bool  __stdcall  RunDebugger	(const char* nameBrk)	= 0;
};

//////////////////////////////////////////////////////////////////////////

// :-( I need copy this wrappers for gcc (MinGW studies compiler) ...
namespace boost_copy_paste
{
	template<class T> class reference_wrapper
	{ 
	public:
		typedef T type;
		explicit reference_wrapper(T& t): t_(&t) {}
		operator T& () const { return *t_; }
		T& get() const { return *t_; }
		T* get_pointer() const { return t_; }
	private:
		T* t_;
	};

	template<class T> inline reference_wrapper<T> const ref(T & t) { return reference_wrapper<T>(t); }
	template<class T> inline reference_wrapper<T const> const cref(T const & t) { return reference_wrapper<T const>(t); }
}

template<class TObject, class TRetVal=TObject>
struct __s_elapi_object_factory 
{

#define __ELAPI_ALLOCATOR_CREATE__BODY__(__TObj, __master__) \
	void*const _ptr= __master__ .GetVarFactory().GetUserStudiesAllocator().allocate(sizeof(__TObj));	\
	return new (_ptr) 

	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject();
	}

	template<class TParam1>
	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master, TParam1 _param1)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject(_param1);
	}

	template<class TParam1, class TParam2>
	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master, TParam1 _param1, TParam2 _param2)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject(_param1,_param2);
	}

	template<class TParam1, class TParam2, class TParam3>
	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master, TParam1 _param1, TParam2 _param2, TParam3 _param3)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject(_param1,_param2,_param3);
	}

	template<class TParam1, class TParam2, class TParam3, class TParam4>
	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master, TParam1 _param1, TParam2 _param2, TParam3 _param3, TParam4 _param4)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject(_param1,_param2,_param3,_param4);
	}

	template<class TParam1, class TParam2, class TParam3, class TParam4, class TParam5>
	static inline TRetVal*	elapi_allocator_create(IBaseStudy& _master, TParam1 _param1, TParam2 _param2, TParam3 _param3, TParam4 _param4, TParam5 _param5)
	{
		__ELAPI_ALLOCATOR_CREATE__BODY__(TObject,_master) TObject(_param1,_param2,_param3,_param4,_param5);
	}

#undef __ELAPI_ALLOCATOR_CREATE__BODY__

	static inline void elapi_allocator_destroy(IBaseStudy& _master, TObject*const _pobj)
	{
		if (_pobj)
			_pobj->~TObject();
		_master.GetVarFactory().GetUserStudiesAllocator().deallocate(_pobj);
	}
};

//////////////////////////////////////////////////////////////////////////

template<class TStudy>
inline IUserBaseStudy* gcc_create_study(IBaseStudy*const _master, void*const _parent, size_t _flags)
{
	return __s_elapi_object_factory<TStudy,IUserBaseStudy>::elapi_allocator_create(*_master,_master,_parent,_flags);
}

template<class TStudy>
inline void gcc_delete_study(IBaseStudy*const _master, IUserBaseStudy*const _study)
{
	TStudy*const _pstudy_t=(TStudy*)_study;
	__s_elapi_object_factory<TStudy>::elapi_allocator_destroy(*_master,_pstudy_t);
}

template<class TFunction>
inline IFunctionBase* gcc_create_function(IBaseStudy*const _master, void*const _parent, size_t _flags)
{
	return __s_elapi_object_factory<TFunction,IFunctionBase>::elapi_allocator_create(*_master,_master,_parent,_flags);
}

template<class TFunction>
inline void gcc_delete_function(IBaseStudy*const _master, IFunctionBase*const _func)
{
	TFunction*const _pfunc_t=(TFunction*)_func;
	__s_elapi_object_factory<TFunction>::elapi_allocator_destroy(*_master,_pfunc_t);
}

#include "IInputVar.h"
#include "VariableInterfaces.inl"
