#pragma once

#include "VariableInterfaces.h"

template<class TSimple>
class CInputObjectBaseImpl :
	private _InputManage
{
	typedef _InputVal<TSimple>				TInput;
public:
	explicit CInputObjectBaseImpl(IInputsOwner& _master)
		:	m_pOriginalInput(NULL), m_pInput(NULL), m_bb(0)
	{
		_master.push_back(this);
	}
public:
	virtual void*&			__cdecl		access_ptr() OVERRIDE {return (void*&)m_pOriginalInput;}
private:
	virtual _InputValInfo*	__cdecl		get_input() OVERRIDE { return m_pOriginalInput; }
protected:
	virtual void			__cdecl		rebind(const bool _bind) OVERRIDE
	{
		m_pInput=&m_pOriginalInput->GetLink();
		m_bb=0;
		if (_bind)
		{
			const _InputValInfo::EBindedEntity _type=m_pInput->GetEntityInfo();
			switch(_type)
			{
			case _InputValInfo::eBE_Variable :
				m_pInput = &m_pInput->GetBindedInput(m_bb);
				break;
			}
		}
	}
public:
	typename _InputVal<TSimple>::_Result	Value(const real_t _bb=0) { return m_pInput->Value(m_bb+_bb); }
	operator								_InputVal<TSimple>&(){return *m_pOriginalInput;}
public:
	TInput&		get_rebinded_input() const { return *m_pInput; }
	real_t		get_rebinded_input_bb() const { return m_bb; }
protected:
	TInput*		m_pInput;
	real_t		m_bb;
	TInput*		m_pOriginalInput;
};

template<class TSimple>
class CInputObjectBase :
	public CInputObjectBaseImpl<TSimple>
{
	typedef CInputObjectBaseImpl<TSimple>	inherited;
public:
	explicit CInputObjectBase(IInputsOwner& _master)
		:	inherited(_master)
	{}
private:
	virtual void		__cdecl		rebind(const bool _bind) OVERRIDE
	{
		m_is_literal = false;
		inherited::rebind(_bind);
		if (_bind)
		{
			const _InputValInfo::EBindedEntity _type=inherited::m_pInput->GetEntityInfo();
			switch(_type)
			{
			case _InputValInfo::eBE_Constant :
				m_is_literal = true;
				m_literal_value = inherited::m_pInput->Value();
				break;
			}
		}
	}
public:
	typename _InputVal<TSimple>::_Result	Value(const real_t _bb=0)
	{
		return m_is_literal ? m_literal_value : inherited::Value(_bb);
	}
private:
	bool		m_is_literal;
	TSimple		m_literal_value;
};

template<class TSimple>
class CInputObjectBase< IArray<TSimple> > :
	public CInputObjectBaseImpl< IArray<TSimple> >
{
	typedef CInputObjectBaseImpl< IArray<TSimple> >	inherited;
public:
	explicit CInputObjectBase(IInputsOwner& _master)
		:	inherited(_master)
	{}
};

namespace indicator_inputs_traits
{
	template<class TSimple>
	struct inputs_creator_traits;

	template<>
	struct inputs_creator_traits<NumSimple>
	{
		typedef _InputVal<NumSimple>* (__cdecl IVariableFactory::*TCreator)(const char*);
		static inline TCreator	get() { return &IVariableFactory::CreateNumericInputVar; }
	};
	
	template<>
	struct inputs_creator_traits<BoolSimple>
	{
		typedef _InputVal<BoolSimple>* (__cdecl IVariableFactory::*TCreator)(const char*);
		static inline TCreator	get() { return &IVariableFactory::CreateTrueFalseInputVar; }
	};
	
	template<>
	struct inputs_creator_traits<StringSimple>
	{
		typedef _InputVal<StringSimple>* (__cdecl IVariableFactory::*TCreator)(const char*);
		static inline TCreator	get() { return &IVariableFactory::CreateStringInputVar; }
	};
};

template <class TSimple>
class CIndicatorInputVar :
	public CInputObjectBase<TSimple>
{
	typedef typename _InputVal<TSimple>::_Result	TRetType;
	typedef CInputObjectBase<TSimple>				inherited;
public:
	explicit CIndicatorInputVar(IInputsOwner& _master, const char*const _name)
		: inherited(_master)
	{
		typedef typename indicator_inputs_traits::inputs_creator_traits<TSimple>	traits;
		inherited::access_ptr() = (_master.get_host().GetVarFactory().*traits::get())(_name);
	}
};

typedef CIndicatorInputVar<NumSimple>			NumericIndicatorInput;
typedef CIndicatorInputVar<BoolSimple>			TrueFalseIndicatorInput;
typedef CIndicatorInputVar<StringSimple>		StringIndicatorInput;