#ifndef STATIC_ARRAY_H
#define STATIC_ARRAY_H

namespace ELAPITypesHelpers
{
	
	template<class Type, size_t N>
	class CStaticArray
	{
	public:
		typedef size_t			size_type;
		typedef ptrdiff_t		difference_type;
		typedef Type			value_type;
		typedef Type*			pointer;
		typedef const Type*		const_pointer;
		typedef Type&			reference;
		typedef const Type&		const_reference;
	public:
		CStaticArray():m_size(0){}
		CStaticArray(const CStaticArray& _val)
		{
			*this=_val;
		}
	public:

		CStaticArray&	operator=(const CStaticArray& _val)
		{
			for(size_type _i(0), _size(_val.size()); _i<N && _i < _size; ++_i ) 
				m_data[_i]=_val[_i];
			m_size=_val.size();
			return *this; 
		}
	public:
		template<class Iter>
		void			assign(Iter _first, Iter _end)
		{
			const size_type _new_size=std::distance(_first,_end);
			check_size(_new_size);
			for(size_type _i(0); _first!=_end ; ++_first, ++_i)
				m_data[_i]=*_first;
			m_size=_new_size;
		}
	public:
		bool			operator==(const CStaticArray& _val)const
		{
			if(size()==_val.size())
			{
#pragma warning (push)
#pragma warning (disable : 4996)
				return std::equal(begin(),end(),_val.begin());
#pragma warning (pop)
			}
			return false;

		}
	public:
		size_type		max_size() const { return N; }
		size_type		capacity() const { return N; }
		size_type		size() const { return m_size; }
		bool			empty() const { return (0 == size()); }

		void			clear() { m_size=0; }

		void			resize(size_type _new_size, const_reference _val = value_type())
		{
			check_size(_new_size);
			m_size=_new_size;
			for(size_type _i(0), _size(size()); _i < _size; ++_i )  m_data[_i]=_val;
		}

		void			push_back(const_reference _val)
		{
			check_size(size()+1);
			++m_size;
			m_data[m_size-1]=_val;
		}

		const_reference	at(size_type _idx) const 
		{ 
			check_bounds(_idx);
			return m_data[_idx];
		}

		reference		at(size_type _idx)
		{ 
			check_bounds(_idx);
			return m_data[_idx];
		}

		pointer			begin() { return &m_data[0]; }
		pointer			end() { return &m_data[0]+m_size; }
		const_pointer	begin() const { return &m_data[0]; }
		const_pointer	end() const { return &m_data[0]+m_size; }

		const_reference	operator[](size_type _idx) const
		{ 
#ifdef ASSERT
			ASSERT( N >_idx && m_size >_idx );
#endif
			return m_data[_idx];
		}

		reference		operator[](size_type _idx)
		{ 
#ifdef ASSERT
			ASSERT( N >_idx && m_size >_idx );
#endif
			return m_data[_idx];
		}
	private:
		void			check_size(size_type _new_size) const 
		{
			if (N <= _new_size) throw std::overflow_error("CStaticArray : size more then static value");
		}
		void			check_bounds(size_type _idx) const 
		{
			if ( m_size <= _idx ) throw std::out_of_range("CStaticArray : index out of range");
		}
	private:
		
		value_type	m_data[N];
		size_type	m_size;
	public:
		//friend class boost::serialization::access;
		template<class Archive>
		void save(Archive & ar, const unsigned int version) const;



		template<class Archive>
		void load(Archive & ar, const unsigned int version);

		
		
		template<class Archive>                                          
		void serialize(                                                  
		Archive &ar,                                                 
		const unsigned int file_version                              
		);

		//BOOST_SERIALIZATION_SPLIT_MEMBER()

	};


}


#endif

