#ifndef _VARIABLE_H_289587246_
#define _VARIABLE_H_289587246_

#include "VariableInterfaces.h"
#include "AbstractFunction.h"
#include "IInputVar.h"

template <class TElement>
class ArrDef : public _ArrDef<TElement>
{
public:
	explicit ArrDef(_InputVal<TElement>& _initVal, const size_t _dims_count, ...)
		:	m_init_val(_initVal), m_dims_count(_dims_count),
			m_dims(new size_t[m_dims_count]), m_fulldims(new size_t[m_dims_count+1])
	{
		va_list marker;
		va_start(marker,_dims_count);

		{for(size_t i=0; i<m_dims_count; i++) m_dims[i]=va_arg(marker,size_t)+1;}

		m_fulldims[m_dims_count]=1;
		m_fulldims[m_dims_count-1]=m_dims[m_dims_count-1];
		{for(int i=m_dims_count-2; i>=0; i--) m_fulldims[i] = m_fulldims[i+1]*m_dims[i];}
	}

	~ArrDef()
	{
		delete [] m_dims;
		delete [] m_fulldims;
	}

	_InputVal<TElement>*	__cdecl		init_val(){return &m_init_val;}
	size_t					__cdecl		dims_count()const{return m_dims_count;}
	const size_t*			__cdecl		dims()const{return m_dims;}
	const size_t*			__cdecl		fulldims()const{return m_fulldims;}
private:
	const size_t			m_dims_count;
	size_t*const			m_dims;//[n]
	size_t*const			m_fulldims;//[n+1]
	_InputVal<TElement>&	m_init_val;
};

/************************************************************************/
/*                                                                      */
/************************************************************************/

// function inputs

template<class TSimple>
class CFuncInputWrap :
	public CInputObjectBase<TSimple>
{
	typedef CInputObjectBase<TSimple> inherited;
public:
	explicit CFuncInputWrap(IInputsOwner& _master)
		:	inherited(_master)
	{}
};

template<class TSimple>
class CFuncInputWrap < IArray<TSimple> > :
	public CInputObjectBase< IArray<TSimple> >
{
	typedef CInputObjectBase< IArray<TSimple> > inherited;
public:
	explicit CFuncInputWrap(IInputsOwner& _master)
		:	inherited(_master)
	{}
public:
	CDimensionWrap<TSimple>&				GetDimSize(size_t dim) const
	{
		return
			inherited::m_pInput ?
			inherited::m_pInput->GetDimSize(dim) :
			inherited::m_pOriginalInput->GetDimSize(dim);
	}
};

/************************************************************************/
/* Implementation of dynamic array's manipulators						*/
/************************************************************************/

template<typename TSimple>
inline int	array_compare(IArray<TSimple>& _left, const real_t _left_start, 
						  IArray<TSimple>& _right, const real_t _right_start, 
						  const size_t _count)
{
	return _left.Compare(_right,_left_start,_right_start,_count);
}

template<typename TSimple>
inline int	array_copy(IArray<TSimple>& _from, const real_t _from_start, 
					   IArray<TSimple>& _to, const real_t _to_start, 
					   const size_t _count)
{
	return _to.Copy(_from,_from_start,_to_start,_count);
}

template<typename TSimple>
inline int	array_getmaxindex(IArray<TSimple>& _obj)
{
	return static_cast<int>(_obj.GetDimSize(0).Value());
}

template<typename TSimple>
inline int	array_gettype(IArray<TSimple>& _obj)
{
	return _obj.Type();
}

template<typename TSimple>
inline BoolSimple	array_setmaxindex(IArray<TSimple>& _obj, const real_t _new_max_index)
{
	return _obj.Resize(_new_max_index);
}

template<typename TSimple>
inline int	array_setvalrange(IArray<TSimple>& _obj, const real_t _start, const real_t _end, const TSimple& _val)
{
	return _obj.Fill(_start,_end,_val);
}

template<typename TSimple>
inline int	array_sort(IArray<TSimple>& _obj, const real_t _start, const real_t _end, const BoolSimple _ascending)
{
	return _obj.Sort(_start,_end,_ascending);
}

template<typename TSimple>
inline TSimple	array_sum(IArray<TSimple>& _obj, const real_t _start, const real_t _end)
{
	return _obj.Sum(_start,_end);
}

template<typename TSimple>
inline void	fill_array(IArray<TSimple>& _obj, const TSimple& _val)
{
	_obj.FillEx(_val);
}

inline NumericSimple array_getintegervalue(IArray<NumericSimple>& _obj, const real_t index)
{
	return _obj(index);
} 

inline BoolSimple array_getbooleanvalue(IArray<BoolSimple>& _obj, const real_t index)
{
	return _obj(index);
}

inline NumericSimple array_getfloatvalue(IArray<NumericSimple>& _obj, const real_t index)
{
	return _obj(index);
}

inline StringSimple array_getstringvalue(IArray<StringSimple>& _obj, const real_t index)
{
	return _obj(index);
}

inline void array_setintegervalue(IArray<NumericSimple>& _obj, const real_t index, const real_t value)
{
	_obj(index) = value;
}

inline void array_setbooleanvalue(IArray<BoolSimple>& _obj, const real_t index, const bool value)
{
	_obj(index) = value;
}

inline void array_setfloatvalue(IArray<NumericSimple>& _obj, const real_t index, const real_t value)
{
	_obj(index) = value;
}

inline void array_setstringvalue(IArray<StringSimple>& _obj, const real_t index, const StringSimple value)
{
	_obj(index) = value;
}

/************************************************************************/
/* implementation inputs of functions									*/
/************************************************************************/

template<class T, class _Ty, class TMemFunc, class _ResultType>
class _InputVal_impl_BarPersist_detector :
	public _IInputVal_Life,
	public _InputVal<T>
{
	typedef STLLight::vector<const _InputValInfo*>		TMyInfluences;
protected:
	typedef _InputVal<T>								inherited;
	typedef	typename inherited::_Value					_Value;
	typedef typename inherited::_Result					_Result;
	typedef TMemFunc									TCalcFunc;
public:
	typedef	typename _InputVal<T>::_MyT					_MyT;
public:
	explicit _InputVal_impl_BarPersist_detector(IBaseStudy*const _main, _Ty*const _master, const TCalcFunc _func)
		: m_pMaster(_master), m_main(*_main), m_bar_persist(false), m_func(_func)
	{}
private:
	virtual void		__cdecl		AddBarPersistInfluence(const _InputValInfo*const _infl) OVERRIDE { m_influences.push_back(_infl); }
	virtual void		__cdecl		SetBarPersist(const bool _bar_persist) OVERRIDE { m_bar_persist=_bar_persist; }
	virtual bool		__cdecl		BarPersist()const OVERRIDE {
		bool _result=m_bar_persist;
		for(TMyInfluences::const_iterator _i(m_influences.begin()), _end(m_influences.end()); _result && _i!=_end; ++_i)
			_result = _result && (*_i)->BarPersist();
		return _result;
	}
protected:
	_ResultType				_calc_impl(real_t _bb)
	{
		bool& _ctx_is_default_prop = m_pMaster->m_ctx_is_default;
		const bool _prev_val = _ctx_is_default_prop;
		_ctx_is_default_prop=false;
		_ResultType _res = (m_pMaster->*m_func)(_bb);
		_ctx_is_default_prop=_prev_val;
		return _res;
	}
protected:
	_Ty*const				m_pMaster;
	const TCalcFunc			m_func;
	IBaseStudy&				m_main;
private:
	bool					m_bar_persist;
	TMyInfluences			m_influences;
};

template<class T, class _Ty, class TMemFunc>
class _InputVal_impl_Expr_Base {};

template<class T, class _Ty>
class _InputVal_impl_Expr_Base<T,_Ty,T(_Ty::*)(real_t)> :
	public _InputVal_impl_BarPersist_detector<T,_Ty,T(_Ty::*)(real_t),T>
{
	typedef _InputVal_impl_BarPersist_detector<T,_Ty,T(_Ty::*)(real_t),T>		inherited;
protected:
	typedef typename inherited::_Result										_Result;
	typedef	typename inherited::_Value										_Value;
	typedef typename inherited::TCalcFunc									TCalcFunc;
protected:
	explicit _InputVal_impl_Expr_Base(	IBaseStudy*const _main, _Ty*const _master, const TCalcFunc _func)
		:	inherited(_main,_master,_func)
	{}
protected:
	_Result		_calc(real_t _bb) { return (m_ret.get()=inherited::_calc_impl(_bb)); }
private:
	_dummy_ret_vals::_dummy_ret_vals<_Value,10>	m_ret;
};

template<class T, class _Ty>
class _InputVal_impl_Expr_Base<T,_Ty,T&(_Ty::*)(real_t)> :
	public _InputVal_impl_BarPersist_detector<T,_Ty,T&(_Ty::*)(real_t),T&>
{
	typedef _InputVal_impl_BarPersist_detector<T,_Ty,T&(_Ty::*)(real_t),T&>	inherited;
protected:
	typedef typename inherited::_Result										_Result;
	typedef	typename inherited::_Value										_Value;
	typedef typename inherited::TCalcFunc									TCalcFunc;
protected:
	explicit _InputVal_impl_Expr_Base(	IBaseStudy*const _main, _Ty*const _master, const TCalcFunc _func)
		:	inherited(_main,_master,_func)
	{}
protected:
	_Result		_calc(real_t _bb) { return inherited::_calc_impl(_bb); }
};

template<class T, class _Ty, class TFunc>
class _InputVal_impl_Expr_Simple :
	public _InputVal_impl_Expr_Base<T,_Ty,TFunc>
{
public:
	typedef _InputVal_impl_Expr_Base<T,_Ty,TFunc>		inherited;
	typedef typename inherited::_Result					_Result;
	typedef	typename inherited::_Value					_Value;
public:
	explicit _InputVal_impl_Expr_Simple(IBaseStudy*const _main, _Ty*const _master, const TFunc _func)
		: inherited(_main,_master,_func)
	{}
private:
	virtual _Result		__cdecl		Value(real_t _bb) OVERRIDE { return inherited::_calc(_bb); }
};

template<class T, class _Ty, class TFunc>
class _InputVal_impl_Expr :
	public _InputVal_impl_Expr_Base<T,_Ty,TFunc>
{
public:
	typedef _InputVal_impl_Expr_Base<T,_Ty,TFunc>		inherited;
	typedef typename inherited::_Result					_Result;
	typedef	typename inherited::_Value					_Value;
public:
	explicit _InputVal_impl_Expr(IBaseStudy*const _main, _Ty*const _master, const TFunc _func)
		: inherited(_main,_master,_func)
	{}
private:
	virtual _Result		__cdecl		Value(real_t _bb) OVERRIDE
	{
		const bool _need_set_ctx=(0!=_bb);
		if (_need_set_ctx)
			inherited::m_main.SetContextSimple(_bb);

		_Result _result = inherited::_calc(_bb);

		if (_need_set_ctx)
			inherited::m_main.RestoreContextSimple();

		return _result;
	}
};

//////////////////////////////////////////////////////////////////////////

template<class T, class _Ty>
class _InputVal_impl {};

template<class TSimpleType, class TType = TSimpleType>
class _InputVal_impl_Input :
	public _IInputVal_Life,
	public _InputVal<TType>
{
public:
	typedef _InputVal<TType>					inherited;
	typedef typename inherited::EBindedEntity	EBindedEntity;
	typedef typename inherited::_Result			_Result;
	typedef	typename inherited::_Result_const	_Result_const;
	typedef	typename inherited::_Value			_Value;
	typedef	typename inherited::_MyT			_MyT;
public:
	explicit _InputVal_impl_Input(_MyT& _val, real_t _bb)
		:	m_val(_val.GetLink()),m_bb(_bb)
	{}
private:
	_Result			__cdecl		Value(real_t _bb){return m_val.Value(m_bb+_bb);}
private:
	virtual _MyT&			__cdecl		GetLink()OVERRIDE{return NumSimpleNEqual(0.0,m_bb) ? *this : m_val;}
	virtual bool			__cdecl		BarPersist()const OVERRIDE{return m_val.BarPersist();}
	virtual EBindedEntity	__cdecl		GetEntityInfo()const OVERRIDE{return m_val.GetEntityInfo();}
	virtual _MyT&			__cdecl		GetBindedInput(real_t& _bb)OVERRIDE{
		_MyT& _result=m_val.GetBindedInput(_bb);
		_bb+=m_bb;
		return _result;
	}
protected:
	_MyT&			m_val;
private:
	const real_t		m_bb;
};

template<class T>
class _InputVal_impl< T , _InputVal<T> > :
	public _InputVal_impl_Input<T>
{
public:
	typedef _InputVal_impl_Input<T>				inherited;
	typedef typename inherited::_MyT			_MyT;
public:
	explicit _InputVal_impl(_MyT& _val, real_t _bb)
		:	inherited(_val,_bb)
	{}
};

template<class T>
class _InputVal_impl< T , _InputVal< IArray<T> > > :
	public _InputVal_impl_Input< IArray<T> >
{
public:
	typedef _InputVal_impl_Input< IArray<T> >	inherited;
	typedef typename inherited::_MyT			_MyT;
public:
	explicit _InputVal_impl(_MyT& _val, real_t _bb)
		:	inherited(_val,_bb)
	{}
private:
	virtual CDimensionWrap<T>&	__cdecl		GetDimSize(size_t _i) OVERRIDE { return inherited::m_val.GetDimSize(_i); }
};

template<class T>
class _InputVal_impl< T , CDimensionWrap<T> > :
	public _IInputVal_Life,
	public _InputVal<T>
{
public:
	typedef _InputVal<T>						inherited;
	typedef typename inherited::_Result			_Result;
	typedef	typename inherited::_Result_const	_Result_const;
	typedef	typename inherited::_Value			_Value;
	typedef	typename inherited::_MyT			_MyT;

public:
	explicit _InputVal_impl(CDimensionWrap<T>& _val):m_val(_val){}
private:
	_Result		__cdecl		Value(real_t){return (m_ret.get()=m_val.Value());}
private:
	virtual bool		__cdecl		BarPersist()const OVERRIDE {return true;}
private:
	CDimensionWrap<T>&						m_val;
	_dummy_ret_vals::_dummy_ret_vals<T,10>	m_ret;
};

template<class T>
class _InputVal_impl<T,CBaseUserStudyGlobalData::TGlobalVarProxy> :
	public _IInputVal_Life,
	public _InputVal<T>
{
	typedef CBaseUserStudyGlobalData::TGlobalVarProxy		TAccessor;
public:
	typedef _InputVal<T>									inherited;
	typedef typename inherited::EBindedEntity				EBindedEntity;
	typedef typename inherited::EBindedEntityAccessor		EBindedEntityAccessor;
	typedef typename inherited::_Result						_Result;
	typedef	typename inherited::_Result_const				_Result_const;
	typedef	typename inherited::_Value						_Value;
	typedef	typename inherited::_MyT						_MyT;
public:
	explicit _InputVal_impl(TAccessor& _pAccessor, real_t _bb, size_t _dn)
		: m_pAccessor(_pAccessor), m_bb(_bb), m_dn(_dn)
	{}
private:
	_Result				__cdecl		Value(real_t _bb) { return m_pAccessor.get_var_obj(m_dn).Value(m_bb+_bb); }
private:
	virtual EBindedEntity		__cdecl		GetEntityInfo()const OVERRIDE{return EBindedEntityAccessor::eBE_Variable;}
	virtual _InputVal<T>&		__cdecl		GetBindedInput(real_t& _bb) OVERRIDE{ _bb=m_bb; return m_pAccessor.get_var_obj(m_dn);}
private:
	virtual bool				__cdecl		BarPersist()const OVERRIDE {return true;}
private:
	TAccessor&		m_pAccessor;
	const real_t		m_bb;
	const size_t	m_dn;
};

template<class T>
class _InputVal_impl<T,T> :
	public _IInputVal_Life,
	public _InputVal<T>
{
public:
	typedef _InputVal<T>								inherited;
	typedef typename inherited::EBindedEntity			EBindedEntity;
	typedef typename inherited::EBindedEntityAccessor	EBindedEntityAccessor;
	typedef typename inherited::_Result					_Result;
	typedef	typename inherited::_Result_const			_Result_const;
	typedef	typename inherited::_Value					_Value;
	typedef	typename inherited::_MyT					_MyT;

public:
	explicit _InputVal_impl(const _Value& _val):m_val(_val){}
private:
	_Result				__cdecl		Value(real_t){return m_val;}
private:
	virtual EBindedEntity		__cdecl		GetEntityInfo()const OVERRIDE{return EBindedEntityAccessor::eBE_Constant;}
private:
	virtual bool				__cdecl		BarPersist()const OVERRIDE {return true;}
private:
	_Value		m_val;
};

// for CTX expressions 
// ALARMA - !!! m_dummy !!!
// not used this->Value() in expressions twice or more - unpredictable results could been occurred
template<class T1, class T2>
class _InputVal_impl_Wrap :
	public _IInputVal_Life,
	public _InputVal<T1>
{
public:
	typedef _InputVal<T1>						inherited;
	typedef typename inherited::_Result			_Result;
	typedef	typename inherited::_Result_const	_Result_const;
	typedef	typename inherited::_Value			_Value;
	typedef	typename inherited::_MyT			_MyT;

public:
	explicit _InputVal_impl_Wrap(_InputVal<T2>& _val)
		:	m_val(_val.GetLink())
	{}
private:
	_Result		__cdecl		Value(real_t _bb){ return (m_dummy=m_val.Value(_bb)); }
private:
	// <temporary> - think about it later
	virtual bool		__cdecl		BarPersist()const OVERRIDE {return m_val.BarPersist();}
private:
	_InputVal<T2>&	m_val;
	_Value			m_dummy;
};


template<class T>
class _InputVal_impl<T,IPlot> :
	public _IInputVal_Life,
	public _InputVal<T>
{
public:
	typedef _InputVal<T>						inherited;
	typedef typename inherited::_Result			_Result;
	typedef	typename inherited::_Result_const	_Result_const;
	typedef	typename inherited::_Value			_Value;
	typedef	typename inherited::_MyT			_MyT;

public:
	explicit _InputVal_impl(IPlot*const _pPlot, size_t plotNum, real_t _bb)
		: m_pPlot(_pPlot), m_bb(_bb), m_plotNum(plotNum)
	{}
private:
	_Result		__cdecl		Value(real_t _bb) {return m_pPlot->GetPlotValue(m_plotNum, m_bb + _bb);}
private:
	virtual bool		__cdecl		BarPersist()const OVERRIDE {return false;}
private:
	IPlot * const		m_pPlot;
	const real_t			m_bb;
	const size_t		m_plotNum;
};

template <typename _Type>
inline _InputVal_impl< _Type , CDimensionWrap<_Type> > * create_input(IBaseStudy& _master, CDimensionWrap<_Type>& _Input, real_t)
{
	return __s_elapi_object_factory< _InputVal_impl< _Type , CDimensionWrap<_Type> > >::elapi_allocator_create(_master,boost_copy_paste::ref(_Input));
}

template <typename _Type>
inline _InputVal_impl< _Type , _InputVal<_Type> > * create_input(IBaseStudy& _master, _InputVal<_Type>& _Input, real_t _bb)
{
	return __s_elapi_object_factory< _InputVal_impl< _Type , _InputVal<_Type> > >::elapi_allocator_create(_master,boost_copy_paste::ref(_Input),_bb);
}

template <typename _Type>
inline _InputVal_impl<_Type,_InputVal< IArray<_Type> > > * create_input(IBaseStudy& _master, _InputVal< IArray<_Type> >& _Input, real_t _bb)
{
	return __s_elapi_object_factory< _InputVal_impl< _Type , _InputVal< IArray<_Type> > > >::elapi_allocator_create(_master,boost_copy_paste::ref(_Input),_bb);
}

//////////////////////////////////////////////////////////////////////////

namespace _input_proxies_details
{
	struct	_by_value		{ char		_dummy[1]; };
	struct	_by_reference	{ _by_value _dummy[2]; };

	void			__f(...);

	template<class Type, class _Ty>
	_by_value		__f(Type (_Ty::*)(real_t));

	template<class Type, class _Ty>
	_by_reference	__f(Type& (_Ty::*)(real_t));
};

template <class _Type, class _Ty, class TFunc>
struct _InputVal_impl_Expr_Factory_Base
{
private:
	typedef _InputVal_impl_Expr<_Type,_Ty,TFunc>			value_type;
	typedef _InputVal_impl_Expr_Simple<_Type,_Ty,TFunc>		simple_value_type;
public:
	static inline value_type*			create(IBaseStudy& _master, _Ty*const _this, const TFunc _func)
	{
		return __s_elapi_object_factory<value_type>::elapi_allocator_create(_master,&_master,_this,_func);
	}

	static inline simple_value_type*	create_simple(IBaseStudy& _master, _Ty*const _this, const TFunc _func)
	{
		return __s_elapi_object_factory<simple_value_type>::elapi_allocator_create(_master,&_master,_this,_func);
	}
};

template <class _Type, class _Ty, size_t>
struct _InputVal_impl_Expr_Factory;

template <class _Type, class _Ty>
struct _InputVal_impl_Expr_Factory<_Type,_Ty,sizeof(_input_proxies_details::_by_reference)> :
	public _InputVal_impl_Expr_Factory_Base<_Type,_Ty,_Type& (_Ty::*)(real_t)>
{};

template <class _Type, class _Ty>
struct _InputVal_impl_Expr_Factory<_Type,_Ty,sizeof(_input_proxies_details::_by_value)> :
	public _InputVal_impl_Expr_Factory_Base<_Type,_Ty,_Type (_Ty::*)(real_t)>
{};

//////////////////////////////////////////////////////////////////////////

template <typename _Type>
inline _InputVal<_Type>* static_cast_input_ptr(_InputVal<_Type>*const _Input) { return _Input; }
template <typename _Type>
inline _InputVal< IArray<_Type> >* static_cast_input_ptr(_InputVal< IArray<_Type> >*const _Input) { return _Input; }

template <typename _Type>
inline _InputVal<_Type>& static_cast_input_ref(_InputVal<_Type>& _Input) { return _Input; }
template <typename _Type>
inline _InputVal< IArray<_Type> >& static_cast_input_ref(_InputVal< IArray<_Type> >& _Input) { return _Input; }


//////////////////////////////////////////////////////////////////////////

typedef _ELAPI_::user_vars::CStackVar<NumSimple>					NumericLiteVar;
typedef _ELAPI_::user_vars::CStackVar<BoolSimple>					TrueFalseLiteVar;
typedef _ELAPI_::user_vars::CStackVar<StringSimple>					StringLiteVar;

typedef _ELAPI_::user_vars::CVarObject<NumSimple,false>				NumericSimpleVar;
typedef _ELAPI_::user_vars::CVarObject<BoolSimple,false>			TrueFalseSimpleVar;
typedef _ELAPI_::user_vars::CVarObject<StringSimple,false>			StringSimpleVar;

typedef _ELAPI_::user_vars::CVarObject<NumSimple,true>				NumericSeriesVar;
typedef _ELAPI_::user_vars::CVarObject<BoolSimple,true>				TrueFalseSeriesVar;
typedef _ELAPI_::user_vars::CVarObject<StringSimple,true>			StringSeriesVar;

typedef _ELAPI_::user_vars::CVarObject<IArray<NumSimple>,false>		NumericArrSimpleVar;
typedef _ELAPI_::user_vars::CVarObject<IArray<BoolSimple>,false>	TrueFalseArrSimpleVar;
typedef _ELAPI_::user_vars::CVarObject<IArray<StringSimple>,false>	StringArrSimpleVar;

typedef _ELAPI_::user_vars::CVarObject<IArray<NumSimple>,true>		NumericArrSeriesVar;
typedef _ELAPI_::user_vars::CVarObject<IArray<BoolSimple>,true>		TrueFalseArrSeriesVar;
typedef _ELAPI_::user_vars::CVarObject<IArray<StringSimple>,true>	StringArrSeriesVar;

// functions

typedef IFunction<NumericSimple>&					NumericFunction;
typedef IFunction<BoolSimple>&						BoolFunction;
typedef IFunction<StringSimple>&					StringFunction;

// function inputs

typedef CFuncInputWrap<NumSimple>					NumericVarFuncInput;
typedef CFuncInputWrap<BoolSimple>					TrueFalseVarFuncInput;
typedef CFuncInputWrap<StringSimple>				StringVarFuncInput;

typedef CFuncInputWrap<NumSimple>					NumericVarRefFuncInput;
typedef CFuncInputWrap<BoolSimple>					TrueFalseVarRefFuncInput;
typedef CFuncInputWrap<StringSimple>				StringVarRefFuncInput;

typedef CFuncInputWrap<IArray<NumSimple> >			NumericArrFuncInput;
typedef CFuncInputWrap<IArray<BoolSimple> >			TrueFalseArrFuncInput;
typedef CFuncInputWrap<IArray<StringSimple> >		StringArrFuncInput;
													
typedef CFuncInputWrap<IArray<NumSimple> >			NumericArrRefFuncInput;
typedef CFuncInputWrap<IArray<BoolSimple> >			TrueFalseArrRefFuncInput;
typedef CFuncInputWrap<IArray<StringSimple> >		StringArrRefFuncInput;

struct el_owner_traits
{
	static inline IFunctionBase*	get_base_ptr(IFunctionBase*const _owner){return _owner;}
	static inline IUserBaseStudy*	get_base_ptr(IUserBaseStudy*const _owner){return _owner;}
	static inline IFunctionBase*	get_parent_ptr(IFunctionBase*const _owner){return _owner;}
	static inline IFunctionBase*	get_parent_ptr(IUserBaseStudy*const){return NULL;}
};

//////////////////////////////////////////////////////////////////////////

#define __CREATE_SOME_VARIABLE__(_Create_Func_Name, _Sequence_N,_Attributes,_Name,_Data_N_Binding) \
	*(m_Master->GetVarFactory(). _Create_Func_Name (_Sequence_N, EVariableAttributes( _Attributes | m_flags ), el_owner_traits::get_parent_ptr(this), _Name, _Data_N_Binding)),*this

#define __CREATE_SOME_LITE_VARIABLE__(_Sequence_N, _Attr, _Name)					*this, _Sequence_N, EVariableAttributes(_Attr| m_flags), el_owner_traits::get_parent_ptr(this), _Name

#define CREATE_NUMERIC_VARIABLE(_SeqN,_Attr,_Name,_Data_N)					__CREATE_SOME_VARIABLE__(CreateNumeric,_SeqN,_Attr,_Name,_Data_N)
#define CREATE_BOOL_VARIABLE(_SeqN,_Attr,_Name,_Data_N)						__CREATE_SOME_VARIABLE__(CreateBool,_SeqN,_Attr,_Name,_Data_N)
#define CREATE_STRING_VARIABLE(_SeqN,_Attr,_Name,_Data_N)					__CREATE_SOME_VARIABLE__(CreateString,_SeqN,_Attr,_Name,_Data_N)

#define CREATE_NUMERIC_ARRAY(_SeqN,_Attr,_Name,_Data_N)						__CREATE_SOME_VARIABLE__(CreateNumericArr,_SeqN,_Attr,_Name,_Data_N)
#define CREATE_BOOL_ARRAY(_SeqN,_Attr,_Name,_Data_N)						__CREATE_SOME_VARIABLE__(CreateBoolArr,_SeqN,_Attr,_Name,_Data_N)
#define CREATE_STRING_ARRAY(_SeqN,_Attr,_Name,_Data_N)						__CREATE_SOME_VARIABLE__(CreateStringArr,_SeqN,_Attr,_Name,_Data_N)

#define CREATE_LITE_NUMERIC_VARIABLE(_SeqN,_Attr,_Name,_Data_N)				__CREATE_SOME_LITE_VARIABLE__(_SeqN,_Attr,_Name)
#define CREATE_LITE_BOOL_VARIABLE(_SeqN,_Attr,_Name,_Data_N)				__CREATE_SOME_LITE_VARIABLE__(_SeqN,_Attr,_Name)
#define CREATE_LITE_STRING_VARIABLE(_SeqN,_Attr,_Name,_Data_N)				__CREATE_SOME_LITE_VARIABLE__(_SeqN,_Attr,_Name)

// function inputs creation
#define MAKE_INPUT_FROM_LITERAL(_Type,_Val)									(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl< _Type , _Type > >::elapi_allocator_create(*m_Master, _Val ) )))
#define MAKE_INPUT_FROM_ATOM(_Type,_Input,_bb)								(static_cast_input_ptr(m_InputsHolder.add( create_input<_Type>(*m_Master, _Input, _bb) )) )
#define MAKE_INPUT_FROM_GSERIES(_Type, _Accessor ,_bb,_dn)					(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl< _Type , CBaseUserStudyGlobalData::TGlobalVarProxy> >::elapi_allocator_create(*m_Master, boost_copy_paste::ref( _Accessor ), _bb , _dn ) )))
#define MAKE_INPUT_FROM_FUNCTION(_Type,_Func)								(&static_cast_input_ref< _Type >( _Func ))//(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl< _Type , IFunction< _Type >& > >::elapi_allocator_create(*m_Master,boost_copy_paste::ref( _Func ) ) )))
//#define MAKE_INPUT_FROM_EXPRESSION(_Type,_master_class,_pFunc)				(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl< _Type , _master_class > >::elapi_allocator_create(*m_Master,m_Master,this,& _master_class :: _pFunc ) )))
#define MAKE_INPUT_FROM_EXPRESSION(_Type,_master_class,_pFunc)				(static_cast_input_ptr(m_InputsHolder.add(_InputVal_impl_Expr_Factory<_Type , _master_class, sizeof(_input_proxies_details::__f( & _master_class :: _pFunc )) > ::create(*m_Master,this,& _master_class :: _pFunc ) )))
#define MAKE_INPUT_FROM_EXPRESSION_INNER(_Type,_master_class,_pFunc)		(static_cast_input_ptr(m_InputsHolder.add(_InputVal_impl_Expr_Factory<_Type , _master_class, sizeof(_input_proxies_details::__f( & _master_class :: _pFunc )) > ::create_simple(*m_Master,this,& _master_class :: _pFunc ) )))
#define MAKE_INPUT_FROM_PLOT(plotNum,_bb)									(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl< NumSimple , IPlot > >::elapi_allocator_create(*m_Master,m_pPlots, plotNum, _bb) )))

// function CTX creation
// BB

#define __MAKE_CTX_FROM_ATOM__(_CTX_TYPE_,_Type,_Input,_bb)					(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl_Wrap<_CTX_TYPE_,_Type> >::elapi_allocator_create(*m_Master, boost_copy_paste::ref( * MAKE_INPUT_FROM_ATOM(_Type,_Input,_bb)) )) ))
#define __MAKE_CTX_FROM_GSERIES__(_CTX_TYPE_,_Type, _Accessor,_bb,_dn)		(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl_Wrap<_CTX_TYPE_,_Type> >::elapi_allocator_create(*m_Master, boost_copy_paste::ref( * MAKE_INPUT_FROM_GSERIES(_Type, _Accessor,_bb,_dn)) )) ))
#define __MAKE_CTX_FROM_FUNCTION__(_CTX_TYPE_,_Type,_Func)					(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl_Wrap<_CTX_TYPE_,_Type> >::elapi_allocator_create(*m_Master, boost_copy_paste::ref( * MAKE_INPUT_FROM_FUNCTION(_Type,_Func)) )) ))
#define __MAKE_CTX_FROM_EXPRESSION__(_CTX_TYPE_,_Type,_master_class,_pFunc)	(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl_Wrap<_CTX_TYPE_,_Type> >::elapi_allocator_create(*m_Master, boost_copy_paste::ref( * MAKE_INPUT_FROM_EXPRESSION(_Type,_master_class,_pFunc)) )) ))

#define _BB_TYPE_ real_t
#define MAKE_BB_FROM_ATOM(_Type,_Input,_bb)									__MAKE_CTX_FROM_ATOM__(_BB_TYPE_,_Type,_Input,_bb)
#define MAKE_BB_FROM_GSERIES(_Type, _Accessor,_bb,_dn)						__MAKE_CTX_FROM_GSERIES__(_BB_TYPE_,_Type, _Accessor,_bb,_dn)
#define MAKE_BB_FROM_FUNCTION(_Type,_Func)									__MAKE_CTX_FROM_FUNCTION__(_BB_TYPE_,_Type,_Func)
#define MAKE_BB_FROM_EXPRESSION(_Type,_master_class,_pFunc)					__MAKE_CTX_FROM_EXPRESSION__(_BB_TYPE_,_Type,_master_class,_pFunc)

// DN
#define _DN_TYPE_ size_t
#define MAKE_DN_VARIABLE_DEFAULT()											(static_cast_input_ptr(m_InputsHolder.add( __s_elapi_object_factory< _InputVal_impl<_DN_TYPE_,_DN_TYPE_> >::elapi_allocator_create(*m_Master,0) )))
#define MAKE_DN_FROM_ATOM(_Type,_Input,_bb)									__MAKE_CTX_FROM_ATOM__(_DN_TYPE_,_Type,_Input,_bb)
#define MAKE_DN_FROM_GSERIES(_Type, _Accessor,_bb,_dn)						__MAKE_CTX_FROM_GSERIES__(_DN_TYPE_,_Type, _Accessor,_bb,_dn)
#define MAKE_DN_FROM_FUNCTION(_Type,_Func)									__MAKE_CTX_FROM_FUNCTION__(_DN_TYPE_,_Type,_Func)
#define MAKE_DN_FROM_EXPRESSION(_Type,_master_class,_pFunc)					__MAKE_CTX_FROM_EXPRESSION__(_DN_TYPE_,_Type,_master_class,_pFunc)

// vars initer's
#define MAKE_INITER_FROM_LITERAL(_Type,_Val)								*MAKE_INPUT_FROM_LITERAL(_Type,_Val)
#define MAKE_INITER_FROM_ATOM(_Type,_Input,_bb)								*MAKE_INPUT_FROM_ATOM(_Type,_Input,_bb)
#define MAKE_INITER_FROM_GSERIES(_Type, _Accessor,_bb,_dn)					*MAKE_INPUT_FROM_GSERIES(_Type, _Accessor,_bb,_dn)
#define MAKE_INITER_FROM_FUNCTION(_Type,_Func)								*MAKE_INPUT_FROM_FUNCTION(_Type,_Func)
#define MAKE_INITER_FROM_EXPRESSION(_Type,_master_class,_pFunc)				*MAKE_INPUT_FROM_EXPRESSION(_Type,_master_class,_pFunc)
#define MAKE_INITER_FROM_PLOT(plotNum,_bb)									*MAKE_INPUT_FROM_PLOT(plotNum,_bb)

// functions creation

#define CREATE_USER_FUNCTION(_Type,_FuncName,_Flags)						(*((IFunction< _Type >*)(m_Master->CreateFuncObj( #_FuncName, el_owner_traits::get_base_ptr(this), m_flags | _Flags))))

inline void __SET_FUNCTION_PARENT_IMPL_BASE(IFunctionBase*const _child, IFunctionBase*const _parent, IBaseStudy* _baseInd)
{
	_baseInd->GetVarFactory().SetParent(_child,_parent);
}

template<typename T1, typename T2>
inline void __SET_FUNCTION_PARENT_IMPL(IFunction<T1>& _child, IFunction<T2>& _parent, IBaseStudy* _baseInd)
{
	__SET_FUNCTION_PARENT_IMPL_BASE(&_child,&_parent,_baseInd);
}

template<typename T>
inline void __SET_FUNCTION_OPTIMISATION_IMPL(IFunction<T>& _func, const EFunctionAttributes _val, IBaseStudy* _baseInd)
{
	_baseInd->GetVarFactory().SetOptimisationAttributes(static_cast<IFunctionBase*>(&_func),_val);
}
/*
template<typename T>
inline void SET_FUNCTION_BAR_PERSIST_INPUT_INFLUENCE(IFunction<T>& _func, const size_t _inp_idx)
{
	_func.push_influence_input(_inp_idx);
}
*/
template<typename T>
inline void SET_INPUT_BAR_PERSIST(IFunction<T>& _func, const size_t _inp_idx, const bool _bar_persist)
{
	_func.access_influence_for_input(_inp_idx)->SetBarPersist(_bar_persist);
}

template<typename T>
inline void SET_INPUT_BAR_PERSIST_INFLUENCE(IFunction<T>& _func, const size_t _inp_idx, const _InputValInfo& _input)
{
	_func.access_influence_for_input(_inp_idx)->AddBarPersistInfluence(&_input);
}

template<typename T>
inline void SET_BARSBACK_BAR_PERSIST(IFunction<T>& _func, const bool _bar_persist)
{
	_InputValInfo*const _ptr=_func.access_bb();
	if (_ptr)
		_ptr->SetBarPersist(_bar_persist);
}

template<typename T>
inline void SET_BARSBACK_BAR_PERSIST_INFLUENCE(IFunction<T>& _func, const _InputValInfo& _input)
{
	_InputValInfo*const _ptr=_func.access_bb();
	if (_ptr)
		_ptr->AddBarPersistInfluence(&_input);
}

template<typename T>
inline void SET_DATASTREAM_BAR_PERSIST(IFunction<T>& _func, const bool _bar_persist)
{
	_InputValInfo*const _ptr=_func.access_dn();
	if (_ptr)
		_ptr->SetBarPersist(_bar_persist);
}

template<typename T>
inline void SET_DATASTREAM_BAR_PERSIST_INFLUENCE(IFunction<T>& _func, const _InputValInfo& _input)
{
	_InputValInfo*const _ptr=_func.access_dn();
	if (_ptr)
		_ptr->AddBarPersistInfluence(&_input);
}

#define SET_FUNCTION_PARENT(_child_func,_parent_func)	__SET_FUNCTION_PARENT_IMPL(_child_func,_parent_func,m_Master);
#define SET_FUNCTION_OPTIMISATION(_func,_attribs)		__SET_FUNCTION_OPTIMISATION_IMPL(_func,_attribs,m_Master);

//////////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning (push)
#pragma warning (disable:4311)
#endif

template<class T>
inline size_t	varstartaddr(const _ELAPI_::user_vars::CSimpleVarObjectBase<T>& obj)
{
	return reinterpret_cast<size_t>(&obj.object());
}

template<class T>
inline size_t	varstartaddr(const _ELAPI_::user_vars::CSeriesVarObjectBase<T>& obj)
{
	return reinterpret_cast<size_t>(&obj.object());
}

template<class T>
inline size_t	arraystartaddr(const _ELAPI_::user_vars::CSimpleVarObjectBase< IArray<T> >& obj)
{
	return reinterpret_cast<size_t>(&obj.object());
}

template<class T>
inline size_t	arraystartaddr(const _ELAPI_::user_vars::CSeriesVarObjectBase< IArray<T> >& obj)
{
	return reinterpret_cast<size_t>(&obj.object());
}

#ifdef _MSC_VER
#pragma warning (pop)
#endif

#define varsize(expr) 0
#define arraysize(expr) 0

#endif//_VARIABLE_H_289587246_