#pragma once
// vars
//////////////////////////////////////////////////////////////////////////

namespace _ELAPI_
{
	namespace user_vars
	{
		template<class TSimple>
		class CVarObjectBase
		{
		protected:
			typedef IVariable<TSimple>			var_type;
		protected:
			explicit CVarObjectBase(var_type& _var)
				:	m_Var(_var)
			{}
		public:
			operator		_InputVal<TSimple>&(){return m_Var;}
			var_type&		object()const{return m_Var;}
		protected:
			var_type&	m_Var;
		};

		template<class TSimple>
		class CSimpleVarObjectBase :
			public CVarObjectBase<TSimple>,
			private _EntityManage
		{
		protected:
			typedef CVarObjectBase<TSimple>			inherited;
			typedef typename inherited::var_type	var_type;
			typedef typename var_type::_MyRetT		value_type;
		public:
			explicit CSimpleVarObjectBase(var_type& _var, IInputsOwner& _master)
				:	inherited(_var), m_pVar(NULL)
			{
				_master.register_var(this);
			}
		private:
			virtual void	__cdecl	rebind(const bool _bind) OVERRIDE { m_pVar = _bind ? &inherited::m_Var.Value() : NULL; }
		public:
			value_type&		Value(real_t=0) { return m_pVar ? *m_pVar : inherited::m_Var.Value(); }
		private:
			value_type*	m_pVar;
		};

		template<class TSimple>
		class CSeriesVarObjectBase :
			public CVarObjectBase<TSimple>
		{
		protected:
			typedef CVarObjectBase<TSimple>			inherited;
			typedef typename inherited::var_type	var_type;
			typedef typename var_type::_MyRetT		value_type;
		public:
			explicit CSeriesVarObjectBase(var_type& _var, IInputsOwner&)
				:	inherited(_var)
			{}
		public:
			value_type&		Value(const real_t _bb=0) { return inherited::m_Var.Value(_bb); }
		};

		namespace traits
		{
			template<class TSimple, bool _is_series>
			struct BaseClassDetector;

			template<class TSimple>
			struct BaseClassDetector<TSimple,true> { typedef CSeriesVarObjectBase<TSimple>		TVarObjectBase; };

			template<class TSimple>
			struct BaseClassDetector<TSimple,false> { typedef CSimpleVarObjectBase<TSimple>		TVarObjectBase; };
		}

		template<class TSimple, bool _is_series>
		class CVarObject :
			public traits::BaseClassDetector<TSimple,_is_series>::TVarObjectBase
		{
			typedef typename traits::BaseClassDetector<TSimple,_is_series>::TVarObjectBase	inherited;
			typedef typename inherited::var_type											var_type;
			typedef typename inherited::inherited											var_access;
		public:
			explicit CVarObject(var_type& _var, IInputsOwner& _master)
				:	inherited(_var,_master)
			{}
		public:
			void						Init(_InputVal<TSimple>& _val) { var_access::m_Var.Init(_val); }
			//const TSimple*				OffsetElement(const TSimple* _pval, int _offs) const { return var_access::m_Var.OffsetElement(_pval,_offs); }
		};

		template<class TSimple, bool _is_series>
		class CVarObject< IArray<TSimple> , _is_series > :
			public traits::BaseClassDetector< IArray<TSimple> ,_is_series>::TVarObjectBase
		{
			typedef typename traits::BaseClassDetector< IArray<TSimple> ,_is_series>::TVarObjectBase	inherited;
			typedef typename inherited::var_type														var_type;
			typedef typename inherited::inherited														var_access;
		public:
			explicit CVarObject(var_type& _var, IInputsOwner& _master)
				:	inherited(_var,_master)
			{}
		public:
			void						Init(_ArrDef<TSimple>& _val) { var_access::m_Var.Init(_val); }
			//const TSimple*				OffsetElement(const TSimple* _pval, int _offs, int nSpaceOfs) const { return var_access::m_Var.OffsetElement(_pval,_offs,nSpaceOfs); }
			//size_t						GetDimsCount() const { return var_access::m_Var.GetDimsCount(); }
			CDimensionWrap<TSimple>&	GetDimProxy(size_t _idx) { return var_access::m_Var.GetDimProxy(_idx); }
		};

		namespace traits
		{
			typedef void (__cdecl IVariableFactory::*TRegisterFunc)(_VariableManageBase*const,size_t, const EVariableAttributes, IFunctionBase*,const char*);

			template<class TSimple>
			struct TypeDetector;

			template<>
			struct TypeDetector<NumSimple> { static inline TRegisterFunc get(){ return &IVariableFactory::RegisterNumericVar; } };

			template<>
			struct TypeDetector<BoolSimple> { static inline TRegisterFunc get(){ return &IVariableFactory::RegisterBoolVar; } };

			template<>
			struct TypeDetector<StringSimple> { static inline TRegisterFunc get(){ return &IVariableFactory::RegisterStringVar; } };
		}

		template<class TSimple, class TDerived>
		class CStackVarBase :
			public _InputVal<TSimple>,
			private _VariableManageBase
		{
			typedef _InputVal<TSimple>							inherited;
		protected:
			typedef typename inherited::EBindedEntity			EBindedEntity;
			typedef typename inherited::EBindedEntityAccessor	EBindedEntityAccessor;
			typedef typename inherited::_Result					_Result;
		private:
			typedef typename inherited::_Value					_Value;
		public:
			explicit CStackVarBase(	IInputsOwner& _master, const size_t _seq_number,  const EVariableAttributes _Attr,
				IFunctionBase*const _owner, const char*const _name)
			{
				const traits::TRegisterFunc _func = traits::TypeDetector<TSimple>::get();
				(_master.get_host().GetVarFactory().*_func)(this,_seq_number, _Attr, _owner,_name);
			}
		private:
			virtual _Result				__cdecl Value(real_t) OVERRIDE { return static_cast<TDerived*>(this)->Value();}
		private:
			virtual bool				__cdecl	BarPersist()const OVERRIDE {return false;}
			virtual EBindedEntity		__cdecl	GetEntityInfo()const OVERRIDE {return EBindedEntityAccessor::eBE_Variable;}
			virtual void* 				__cdecl	GetAsInputValBaseT() OVERRIDE { return static_cast<_InputValBase<_Value>*>(this); }
		};

		template<class TSimple>
		class CStackVar :
			public CStackVarBase< TSimple,CStackVar<TSimple> >
		{
			typedef CStackVarBase< TSimple,CStackVar<TSimple> >	inherited;
			typedef typename inherited::EBindedEntity			EBindedEntity;
			typedef typename inherited::EBindedEntityAccessor	EBindedEntityAccessor;
			typedef typename inherited::_Result					_Result;
		public:
			explicit CStackVar(	IInputsOwner& _master, const size_t _seq_number,  const EVariableAttributes _Attr,
				IFunctionBase*const _owner, const char*const _name)
				: inherited(_master,_seq_number,_Attr,_owner,_name), m_refDefVal(NULL)
			{}
		public:
			void					Init(_InputVal<TSimple>& _val) { m_refDefVal=&_val; }
			_Result					Value(const int=0) { return m_value; }
		private:
			virtual void			__cdecl	DelayInitialize() OVERRIDE {
				m_value = m_refDefVal ? m_refDefVal->Value() : TSimple();
			}
		private:
			TSimple				m_value;
			_InputVal<TSimple>* m_refDefVal;
		};
	};
}