#pragma once

#ifdef _STUDY_RUNNER_
#define _STUDY_RUNNER_EXPORT_ATTRIBUTE_ __declspec(dllexport)
#else
#define _STUDY_RUNNER_EXPORT_ATTRIBUTE_ __declspec(dllimport)
#endif

#ifndef _MSC_VER
	#define NULL			0
	typedef unsigned int	size_t;
	typedef int				ptrdiff_t;
#else
	#include <crtdefs.h>
#endif

extern "C" _STUDY_RUNNER_EXPORT_ATTRIBUTE_	void * 	__cdecl _EL_malloc(size_t);
extern "C" _STUDY_RUNNER_EXPORT_ATTRIBUTE_	void 	__cdecl _EL_free(void*);
extern "C" _STUDY_RUNNER_EXPORT_ATTRIBUTE_	void 	__cdecl _EL_Xran();
extern "C" _STUDY_RUNNER_EXPORT_ATTRIBUTE_	void 	__cdecl _EL_Xlen();

#ifndef _MSC_VER

extern "C"	void *	__cdecl memcpy(void*,const void*,size_t);
extern "C"	size_t 	__cdecl strlen(const char*);
extern "C"	void * 	__cdecl memset(void*,int,size_t);
extern "C"	int 	__cdecl strcmp(const char*,const char*);
extern "C"	int		__cdecl memcmp(const void*,const void*,size_t);
extern "C"	void *	__cdecl memmove(void*,const void*,size_t);

inline void*	__cdecl operator new(size_t _N) { return _EL_malloc(_N); }
inline void		__cdecl operator delete(void * _Ptr) { _EL_free(_Ptr); }
inline void *	__cdecl operator new(size_t _N, void *_Where) { return _Where; }
inline void		__cdecl operator delete(void *, void *) { }

inline void *	__cdecl operator new[](size_t _N) { return _EL_malloc(_N); }
inline void		__cdecl operator delete[](void * _Ptr) { _EL_free(_Ptr); }
inline void *	__cdecl operator new[](size_t, void *_Where) { return _Where; }
inline void		__cdecl operator delete[](void *, void *) {}

#endif

namespace STLLight
{
	template<class _Type>
	class _EL_allocator
	{
	public:
		typedef _Type				value_type;

		typedef value_type*			pointer;
		typedef value_type&			reference;
		typedef const value_type*	const_pointer;
		typedef const value_type&	const_reference;

		typedef size_t				size_type;
		typedef ptrdiff_t			difference_type;

		template<class _Other>
		struct rebind
		{
			typedef _EL_allocator<_Other> other;
		};


		_EL_allocator()
		{	// construct default allocator (do nothing)
		}

		_EL_allocator(const _EL_allocator<_Type>&)
		{	// construct by copying (do nothing)
		}

		_EL_allocator& operator=(const _EL_allocator&)
		{	// assign from a related allocator (do nothing)
			return (*this);
		}

		void deallocate(pointer _Ptr, size_type)
		{
			_EL_free(_Ptr);
		}

		pointer allocate(size_type _Count)
		{
			return (pointer)_EL_malloc(_Count*sizeof(value_type));
		}

		pointer allocate(size_type _Count, const void* )
		{	// allocate array of _Count elements, ignore hint
			return (allocate(_Count));
		}

		void construct(pointer _Ptr, const _Type& _Val)
		{
			new ((void*)_Ptr) value_type(_Val);
		}

		void destroy(pointer _Ptr)
		{
			if(_Ptr) _Ptr->~value_type();
		}

		size_type max_size() const
		{	// estimate maximum array size
			size_type _Count = (size_type)(-1) / sizeof (value_type);
			return (0 < _Count ? _Count : 1);
		}
	};
};

#ifndef _MSC_VER

	#define _TRY_BEGIN	{{
	#define _CATCH(x)	} if (0) {
	#define _CATCH_ALL	} if (0) {
	#define _CATCH_END	}}
	#define _RERAISE
	#define _STDEXT ::STLLight::
	#define _STD ::STLLight::

	namespace STLLight
	{
		template<class _Ty>
		inline void swap(_Ty& _Left, _Ty& _Right)
		{
			_Ty _Tmp = _Left;
			_Left = _Right, _Right = _Tmp;
		}

		template<class _InIt, class _OutIt>
		inline _OutIt _Copy_opt(_InIt _First, _InIt _Last, _OutIt _Dest)
		{
			for (; _First != _Last; ++_Dest, ++_First)
				*_Dest = *_First;
			return (_Dest);
		}

		template<class _RanIt, class _Diff>
		inline void _Distance(_RanIt _First, _RanIt _Last, _Diff& _Off)
		{
			_Off += _Last - _First;
		}

		template<class _BidIt1, class _BidIt2>
		inline _BidIt2 _Unchecked_move_backward(_BidIt1 _First, _BidIt1 _Last, _BidIt2 _Dest) 
		{
			while (_First != _Last)
				swap(*--_Dest, *--_Last);
			return (_Dest);
		}

		template<class _RI, class _Diff>
		inline void _Advance(_RI& _Where, _Diff _Off)
		{
			_Where += _Off;
		}

		template<class _InIt, class _OutIt>
		inline _OutIt unchecked_copy(_InIt _First, _InIt _Last, _OutIt _Dest)
		{
			return _Copy_opt(_First,_Last,_Dest);
		}

		template<class _FwdIt, class _Ty> inline
			void _Fill(_FwdIt _First, _FwdIt _Last, const _Ty& _Val)
		{
			for (; _First != _Last; ++_First)
				*_First = _Val;
		}

		template<class _Ty, class _Alloc>
		inline void _Destroy_range(_Ty *_First, _Ty *_Last, _Alloc& _Al)
		{
			for (; _First != _Last; ++_First)
				_Al.destroy(_First);
		}

		template<class _FwdIt, class _Diff, class _Tval, class _Alloc>
		inline void _Uninit_fill_n(_FwdIt _First, _Diff _Count, const _Tval& _Val, _Alloc& _Al)
		{
			_FwdIt _Next = _First;
			for (; 0 < _Count; --_Count, ++_First)
				_Al.construct(&*_First, _Val);
		}

		template<class _FwdIt, class _Diff, class _Tval, class _Alloc>
		inline void unchecked_uninitialized_fill_n(_FwdIt _First, _Diff _Count, const _Tval& _Val, _Alloc& _Al)
		{
			_Uninit_fill_n(_First, _Count, _Val, _Al);
		}

		template<class _InIt, class _FwdIt, class _Alloc>
		inline _FwdIt _Uninit_copy(_InIt _First, _InIt _Last, _FwdIt _Dest, _Alloc& _Al)
		{	
			_FwdIt _Next = _Dest;
			for (; _First != _Last; ++_Dest, ++_First)
				_Al.construct(_Dest, *_First);
			return (_Dest);
		}

		template<class _InIt, class _FwdIt, class _Alloc>
		inline _FwdIt unchecked_uninitialized_copy(_InIt _First, _InIt _Last, _FwdIt _Dest, _Alloc& _Al)
		{
			return _STD _Uninit_copy(_First, _Last, _Dest, _Al);
		}

		template<class _InIt, class _FwdIt, class _Alloc>
		inline _FwdIt _Uninit_move(_InIt _First, _InIt _Last, _FwdIt _Dest, _Alloc& _Al)
		{
			_FwdIt _Next = _Dest;
			typename _Alloc::value_type _Val;
			for (; _First != _Last; ++_Dest, ++_First)
			{
				_Al.construct(_Dest, _Val);
				_STD swap(*_Dest, *_First);
			}
			return (_Dest);
		}

		template<class _InIt, class _FwdIt, class _Alloc>
		inline _FwdIt _Unchecked_uninitialized_move(_InIt _First, _InIt _Last, _FwdIt _Dest, _Alloc& _Al)
		{
			return _STD _Uninit_move(_First, _Last, _Dest, _Al);
		}

		template<class _InIt1, class _InIt2, class _Pr>
		inline bool lexicographical_compare(_InIt1 _First1, _InIt1 _Last1, _InIt2 _First2, _InIt2 _Last2)
		{
			for (; _First1 != _Last1 && _First2 != _Last2; ++_First1, ++_First2)
				if (*_First1 < *_First2)
					return (true);
				else if (*_First2 < *_First1)
					return (false);
			return (_First1 == _Last1 && _First2 != _Last2);
		}

		template<class _InIt1, class _InIt2>
		inline bool equal(_InIt1 _From_first, _InIt1 _To_first, _InIt2 _From_second)
		{
			for(;_From_first!=_To_first; ++_From_first, ++_From_second)
				if (*_From_first != *_From_second)
					return false;
			return true;
		}

	};

#else

	#include <stddef.h>
	#include <memory>
	#include <exception>

#endif
