#pragma once

#include "ValWrapper.h"
#include "VariableInterfaces.h"
#include "IExpIndicator.h"

class CIntrisincsWrappers
{
public:
	explicit CIntrisincsWrappers(intrinsic_functions::IntrinsicCtx& _ctx)
		: m_ctx(_ctx)
	{}
private:
	template<class TSimple>
	class CObjWrapper
	{
	public:
		CObjWrapper(_MRetVal<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(IVariable<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(_InputVal<TSimple>& _obj)
			:	m_Obj( _obj ), m_bb(0)
		{}

		CObjWrapper(const CInputObjectBase<TSimple>& _obj)
			:	m_Obj(_obj.get_rebinded_input()), m_bb(_obj.get_rebinded_input_bb())
		{}
	public:
		_InputVal<TSimple>&	get_object()const{return m_Obj;}
		real_t				get_bb()const{return m_bb;}
	private:
		_InputVal<TSimple>&	m_Obj;
		const real_t			m_bb;
	};

	template<class TSimple>
	class CLiteralParamWrapper
	{
	public:
		CLiteralParamWrapper(const TSimple& _val)
			:	Value(_val)
		{}

		CLiteralParamWrapper(_MRetVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CLiteralParamWrapper(IVariable<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CLiteralParamWrapper(_InputVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CLiteralParamWrapper(CInputObjectBase<TSimple>& _obj)
			:	Value(_obj.Value())
		{}
	public:
		const TSimple	Value;
	};

	template<class TSimple>
	class CRefParamWrapper
	{
	public:
		CRefParamWrapper(TSimple& _val)
			:	Value(_val)
		{}

		CRefParamWrapper(_MRetVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CRefParamWrapper(IVariable<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CStackVar<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CSimpleVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_ELAPI_::user_vars::CSeriesVarObjectBase<TSimple>& _obj)
			:	Value( _obj.Value() )
		{}

		CRefParamWrapper(_InputVal<TSimple>& _obj)
			:	Value(_obj.Value())
		{}

		CRefParamWrapper(CInputObjectBase<TSimple>& _obj)
			:	Value(_obj.Value())
		{}
	public:
		TSimple&	Value;
	};

public:
	NumSimple	intrinsic_summation(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).summ(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_average(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return intrinsic_summation(_obj,_lenght)/_lenght.Value;
	}

	void		intrinsic_extremes(	const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght, 
									const CLiteralParamWrapper<NumSimple>& _min_or_max/*-1 - min, 1 - max*/,
									const CRefParamWrapper<NumSimple>& _result, const CRefParamWrapper<NumSimple>& _bb_offset)
	{
		get_interface( _obj.get_object() ).extremes(m_ctx,_min_or_max.Value,_obj.get_bb(),
													_lenght.Value,_result.Value,_bb_offset.Value);
	}

	NumSimple	intrinsic_waverage(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).waverage(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_varianceps(const CObjWrapper<NumSimple>& _obj,
									 const CLiteralParamWrapper<NumSimple>& _lenght,
									 const CLiteralParamWrapper<NumSimple>& _data_type)
	{
		return get_interface( _obj.get_object() ).varianceps(m_ctx,_data_type.Value,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_ulcerindex(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).ulcerindex(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_summationif(	const CObjWrapper<NumSimple>& _obj,
										const CObjWrapper<BoolSimple>& _test,
										const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).summif(m_ctx,_test.get_object(),_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_pivot(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght,
								const CLiteralParamWrapper<NumSimple>& _leftstrength,
								const CLiteralParamWrapper<NumSimple>& _rightstrength,
								const CLiteralParamWrapper<NumSimple>& _instance,
								const CLiteralParamWrapper<NumSimple>& _min_or_max/*-1 - min, 1 - max*/,
								const CRefParamWrapper<NumSimple>& _oPivotprice,
								const CRefParamWrapper<NumSimple>& _oPivot_bb_offset)
	{
		return get_interface( _obj.get_object() ).pivot(m_ctx,_obj.get_bb(),_lenght.Value,_min_or_max.Value,
					_leftstrength.Value,_rightstrength.Value,_instance.Value,_oPivotprice.Value,_oPivot_bb_offset.Value);
	}

	NumSimple	intrinsic_linearreg(const CObjWrapper<NumSimple>& _obj,
									const CLiteralParamWrapper<NumSimple>& _lenght,
									const CLiteralParamWrapper<NumSimple>& _tgt_bar,
									const CRefParamWrapper<NumSimple>& _oLR_Slope,
									const CRefParamWrapper<NumSimple>& _oLR_Angle,
									const CRefParamWrapper<NumSimple>& _oLR_Intercept,
									const CRefParamWrapper<NumSimple>& _oLR_Value)
	{
		return get_interface( _obj.get_object() ).linear_reg(m_ctx,_obj.get_bb(),_lenght.Value,
											_tgt_bar.Value,_oLR_Slope.Value,_oLR_Angle.Value,_oLR_Intercept.Value,_oLR_Value.Value);
	}

	NumSimple	intrinsic_stderror(const CObjWrapper<NumSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).std_error(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	NumSimple	intrinsic_countif(const CObjWrapper<BoolSimple>& _obj, const CLiteralParamWrapper<NumSimple>& _lenght)
	{
		return get_interface( _obj.get_object() ).count_if(m_ctx,_obj.get_bb(),_lenght.Value);
	}

	void	intrinsic_2Darr_copy_column(NumSimple*const _arr, const size_t _start_row, const size_t _ind1,
		const size_t _ind2,	const size_t _size1, const size_t _size2)
	{
		for (size_t _i = _start_row; _i <= _size1; ++_i) {
			_arr[_i*_size2+_ind1] = _arr[_i*_size2+_ind2];
		}
	}

	bool	intrinsic_heappush2D(NumSimple*const _arr, const size_t _size,
		const size_t _order, const size_t _arr_size, size_t* _ind_ref) {
		
		if ( *_ind_ref < _size ) {

			size_t _colindx4 = *_ind_ref + 1;

			if (_arr[_arr_size+_colindx4] > _arr[_arr_size+(*_ind_ref)] && _order == -1 ||
				_arr[_arr_size+_colindx4] < _arr[_arr_size+(*_ind_ref)] && _order == 1) 
					*_ind_ref = _colindx4 ;
		} 

		return (_order == -1) ? _arr[_arr_size] < _arr[_arr_size+(*_ind_ref)] : _arr[_arr_size] > _arr[_arr_size+(*_ind_ref)];
	}

	void	intrinsic_heapsift2D(NumSimple*const _arr, size_t _ind, size_t _size1,
		size_t _size2, size_t _order, size_t _arr_size2) {

		size_t _ind2 = _ind;
		size_t _ind3 = 2*_ind2;

		while ( _ind3 <= _size2 &&
			intrinsic_heappush2D(_arr, _size2, _order, _arr_size2, &_ind3) ) 
		{
			intrinsic_2Darr_copy_column(_arr, 1, _ind2, _ind3, _size1, _arr_size2);

			_ind2 = _ind3;
			_ind3 = 2*_ind2;
		}

		intrinsic_2Darr_copy_column(_arr, 1, _ind2, 0, _size1, _arr_size2);
	}
private:
	template<class TSimple>
	const intrinsic_functions::IntrinsicFunctions<TSimple>&	get_interface(_InputVal<TSimple>& _obj)
	{
		const intrinsic_functions::IntrinsicFunctions<TSimple>*const _ptr=_obj.GetIntrinsics();
		return _ptr ? *_ptr : m_ctx.base_study().GetIntrinsicAccessor().GetIntrinsics(_obj);
	}
private:
	intrinsic_functions::IntrinsicCtx&	m_ctx;
};