#pragma once

#include "common_Light.h"

namespace STLLight
{

#pragma pack(push,8)
#pragma warning(push,3)

#pragma warning(disable: 4189)
	
	struct char_traits
	{	// properties of a string or stream char element
		typedef char _Elem;
		typedef _Elem char_type;
		typedef int int_type;

		static void __cdecl assign(_Elem& _Left, const _Elem& _Right)
		{	// assign an element
			_Left = _Right;
		}

		static bool __cdecl eq(const _Elem& _Left, const _Elem& _Right)
		{	// test for element equality
			return (_Left == _Right);
		}

		static bool __cdecl lt(const _Elem& _Left, const _Elem& _Right)
		{	// test if _Left precedes _Right
			return (_Left < _Right);
		}

		static int __cdecl compare(const _Elem *_First1, const _Elem *_First2, size_t _Count)
		{	// compare [_First1, _First1 + _Count) with [_First2, ...)
			return (::memcmp(_First1, _First2, _Count*sizeof(_Elem)));
		}

		static size_t __cdecl length(const _Elem *_First)
		{	// find length of null-terminated string
			return (::strlen(_First));
		}

		static _Elem *__cdecl copy(_Elem *_First1, const _Elem *_First2,
			size_t _Count)
		{	// copy [_First1, _First1 + _Count) to [_First2, ...)
			return ((_Elem *)::memcpy(_First1, _First2, _Count*sizeof(_Elem)));
		}

		static _Elem *__cdecl move(_Elem *_First1, const _Elem *_First2,
			size_t _Count)
		{	// copy [_First1, _First1 + _Count) to [_First2, ...)
			return ((_Elem *)::memmove(_First1, _First2, _Count*sizeof(_Elem)));
		}

		static _Elem *__cdecl assign(_Elem *_First, size_t _Count, _Elem _Ch)
		{	// assign _Count * _Ch to [_First, ...)
			return ((_Elem *)::memset(_First, _Ch, _Count*sizeof(_Elem)));
		}

		static _Elem __cdecl to_char_type(const int_type& _Meta)
		{	// convert metacharacter to character
			return ((_Elem)_Meta);
		}

		static int_type __cdecl to_int_type(const _Elem& _Ch)
		{	// convert character to metacharacter
			return ((unsigned char)_Ch);
		}

		static bool __cdecl eq_int_type(const int_type& _Left,
			const int_type& _Right)
		{	// test for metacharacter equality
			return (_Left == _Right);
		}

		static int_type __cdecl eof()
		{	// return end-of-file metacharacter
			return (-1);
		}

		static int_type __cdecl not_eof(const int_type& _Meta)
		{	// return anything but EOF
			return (_Meta != eof() ? _Meta : !eof());
		}
	};

	class _string
	{
		// null-terminated transparent array of elements
	public:
		typedef _string			_Myt;
		typedef	char			_Elem;
		typedef size_t			size_type;
		typedef ptrdiff_t		difference_type;
		typedef char*			_Tptr;
		typedef const char*		_Ctptr;
		typedef _Tptr			pointer;
		typedef _Ctptr			const_pointer;
		typedef char&			_Reft;
		typedef _Reft			reference;
		typedef const char&		const_reference;
		typedef char			value_type;
		typedef char_traits		_Traits;
		typedef _Traits			traits_type;
	private:
		_EL_allocator<_Elem>	_Alval;
	public:

		_string()
		{	// construct empty string
			_Tidy();
		}

		_string(const _Myt& _Right)
		{	// construct by copying _Right
			_Tidy();
			assign(_Right, 0, npos);
		}

		_string(const _Myt& _Right, size_type _Roff, size_type _Count = npos)
		{	// construct from _Right [_Roff, _Roff + _Count)
			_Tidy();
			assign(_Right, _Roff, _Count);
		}

		_string(const _Elem *_Ptr, size_type _Count)
		{	// construct from [_Ptr, _Ptr + _Count)
			_Tidy();
			if (NULL!=_Ptr) assign(_Ptr, _Count);
		}

		_string(const _Elem *_Ptr)
		{	// construct from [_Ptr, <null>)
			_Tidy();
			if (NULL!=_Ptr) assign(_Ptr);
		}

		_string(size_type _Count, _Elem _Ch)
		{	// construct from _Count * _Ch
			_Tidy();
			assign(_Count, _Ch);
		}

		_string(const_pointer _First, const_pointer _Last)
		{	// construct from [_First, _Last), const pointers
			_Tidy();
			if (_First != _Last)
				assign(&*_First, _Last - _First);
		}

		~_string()
		{	// destroy the string
			_Tidy(true);
		}

		static const size_type npos = (size_type)(-1);	// generic bad/missing length/position

		_Myt& operator=(const _Myt& _Right)
		{	// assign _Right
			return (this!=&_Right) ? (assign(_Right)) : *this ;
		}

		_Myt& operator=(const _Elem *_Ptr)
		{	// assign [_Ptr, <null>)
			return (NULL!=_Ptr) ? (assign(_Ptr)) : *this;
		}

		_Myt& operator=(_Elem _Ch)
		{	// assign 1 * _Ch
			return (assign(1, _Ch));
		}

		_Myt& operator+=(const _Myt& _Right)
		{	// append _Right
			return (append(_Right));
		}

		_Myt& operator+=(const _Elem *_Ptr)
		{	// append [_Ptr, <null>)
			return (NULL!=_Ptr) ? (append(_Ptr)) : *this;
		}

		_Myt& operator+=(_Elem _Ch)
		{	// append 1 * _Ch
			return (append((size_type)1, _Ch));
		}

		_Myt& append(const _Myt& _Right)
		{	// append _Right
			return (append(_Right, 0, npos));
		}

		_Myt& append(const _Myt& _Right,
			size_type _Roff, size_type _Count)
		{	// append _Right [_Roff, _Roff + _Count)
			if (_Right.size() < _Roff)
				_EL_Xran();	// _Roff off end
			size_type _Num = _Right.size() - _Roff;
			if (_Num < _Count)
				_Count = _Num;	// trim _Count to size
			if (npos - _Mysize <= _Count)
				_EL_Xlen();	// result too long

			if (0 < _Count && _Grow(_Num = _Mysize + _Count))
			{	// make room and append new stuff
				_Traits::copy(_Myptr() + _Mysize,
					_Right._Myptr() + _Roff, _Count);
				_Eos(_Num);
			}
			return (*this);
		}

		_Myt& append(const _Elem *_Ptr, size_type _Count)
		{	// append [_Ptr, _Ptr + _Count)
			if (NULL==_Ptr) return (*this);

			if (_Inside(_Ptr))
				return (append(*this, _Ptr - _Myptr(), _Count));	// substring
			if (npos - _Mysize <= _Count)
				_EL_Xlen();	// result too long

			size_type _Num;
			if (0 < _Count && _Grow(_Num = _Mysize + _Count))
			{	// make room and append new stuff
				_Traits::copy(_Myptr() + _Mysize, _Ptr, _Count);
				_Eos(_Num);
			}
			return (*this);
		}

		_Myt& append(const _Elem *_Ptr)
		{	// append [_Ptr, <null>)
			return (append(_Ptr, _Traits::length(_Ptr)));
		}

		_Myt& append(size_type _Count, _Elem _Ch)
		{	// append _Count * _Ch
			if (npos - _Mysize <= _Count)
				_EL_Xlen();	// result too long

			size_type _Num;
			if (0 < _Count && _Grow(_Num = _Mysize + _Count))
			{	// make room and append new stuff using assign
				_Traits::assign(_Myptr() + _Mysize, _Count, _Ch);
				_Eos(_Num);
			}
			return (*this);
		}

		_Myt& assign(const _Myt& _Right)
		{	// assign _Right
			return (assign(_Right, 0, npos));
		}

		_Myt& assign(const _Myt& _Right,
			size_type _Roff, size_type _Count)
		{	// assign _Right [_Roff, _Roff + _Count)
			if (_Right.size() < _Roff)
				_EL_Xran();	// _Roff off end
			size_type _Num = _Right.size() - _Roff;
			if (_Count < _Num)
				_Num = _Count;	// trim _Num to size

			if (this == &_Right)
				erase((size_type)(_Roff + _Num)), erase(0, _Roff);	// substring
			else if (_Grow(_Num))
			{	// make room and assign new stuff
				_Traits::copy(_Myptr(), _Right._Myptr() + _Roff, _Num);
				_Eos(_Num);
			}
			return (*this);
		}

		_Myt& assign(const _Elem *_Ptr, size_type _Num)
		{	// assign [_Ptr, _Ptr + _Num)
			if (NULL==_Ptr) return (*this);

			if (_Inside(_Ptr))
				return (assign(*this, _Ptr - _Myptr(), _Num));	// substring

			if (_Grow(_Num))
			{	// make room and assign new stuff
				_Traits::copy(_Myptr(), _Ptr, _Num);
				_Eos(_Num);
			}
			return (*this);
		}

		_Myt& assign(const _Elem *_Ptr)
		{	// assign [_Ptr, <null>)
			return (assign(_Ptr, _Traits::length(_Ptr)));
		}

		_Myt& assign(size_type _Count, _Elem _Ch)
		{	// assign _Count * _Ch
			if (_Count == npos)
				_EL_Xlen();	// result too long

			if (_Grow(_Count))
			{	// make room and assign new stuff
				_Traits::assign(_Myptr(), _Count, _Ch);
				_Eos(_Count);
			}
			return (*this);
		}

		_Myt& erase(size_type _Off = 0,
			size_type _Count = npos)
		{	// erase elements [_Off, _Off + _Count)
			if (_Mysize < _Off)
				_EL_Xran();	// _Off off end
			if (_Mysize - _Off < _Count)
				_Count = _Mysize - _Off;	// trim _Count
			if (0 < _Count)
			{	// move elements down
				_Traits::move(_Myptr() + _Off, _Myptr() + _Off + _Count,
					_Mysize - _Off - _Count);
				size_type _Newsize = _Mysize - _Count;
				_Eos(_Newsize);
			}
			return (*this);
		}

		void clear()
		{	// erase all
			erase();
		}

		reference operator[](size_type _Off)
		{	// subscript mutable sequence
			return (_Myptr()[_Off]);
		}

		const_reference operator[](size_type _Off) const
		{	// subscript nonmutable sequence
			return (_Myptr()[_Off]);
		}

		const _Elem *c_str() const
		{	// return pointer to null-terminated nonmutable array
			return (_Myptr());
		}

		const _Elem *data() const
		{	// return pointer to nonmutable array
			return (c_str());
		}

		size_type length() const
		{	// return length of sequence
			return (_Mysize);
		}

		size_type size() const
		{	// return length of sequence
			return (_Mysize);
		}

		size_type max_size() const
		{	// return maximum possible length of sequence
			size_type _Num = _Alval.max_size();
			_Num /= 100; // max string lenght ~ 20 Mb
			return (_Num <= 1 ? 1 : _Num - 1);
		}

		void resize(size_type _Newsize)
		{	// determine new length, padding with null elements as needed
			resize(_Newsize, _Elem());
		}

		void resize(size_type _Newsize, _Elem _Ch)
		{	// determine new length, padding with _Ch elements as needed
			if (_Newsize <= _Mysize)
				erase(_Newsize);
			else
				append(_Newsize - _Mysize, _Ch);
		}

		size_type capacity() const
		{	// return current length of allocated storage
			return (_Myres);
		}

		void reserve(size_type _Newcap = 0)
		{	// determine new minimum length of allocated storage
			if (_Mysize <= _Newcap && _Myres != _Newcap)
			{	// change reservation
				size_type _Size = _Mysize;
				if (_Grow(_Newcap, true))
					_Eos(_Size);
			}
		}

		bool empty() const
		{	// test if sequence is empty
			return (_Mysize == 0);
		}

		size_type copy(_Elem *_Ptr,
			size_type _Count, size_type _Off = 0) const
		{	// copy [_Off, _Off + _Count) to [_Ptr, _Ptr + _Count)
			if (_Mysize < _Off)
				_EL_Xran();	// _Off off end
			if (_Mysize - _Off < _Count)
				_Count = _Mysize - _Off;
			_Traits::copy(_Ptr, _Myptr() + _Off, _Count);
			return (_Count);
		}

		_Myt substr(size_type _Off = 0, size_type _Count = npos) const
		{	// return [_Off, _Off + _Count) as new string
			return (_Myt(*this, _Off, _Count));
		}

		int compare(const _Myt& _Right) const
		{	// compare [0, _Mysize) with _Right
			return (compare(0, _Mysize, _Right._Myptr(), _Right.size()));
		}

		int compare(size_type _Off, size_type _N0,
			const _Myt& _Right) const
		{	// compare [_Off, _Off + _N0) with _Right
			return (compare(_Off, _N0, _Right, 0, npos));
		}

		int compare(size_type _Off,
			size_type _N0, const _Myt& _Right,
			size_type _Roff, size_type _Count) const
		{	// compare [_Off, _Off + _N0) with _Right [_Roff, _Roff + _Count)
			if (_Right.size() < _Roff)
				_EL_Xran();	// _Off off end
			if (_Right._Mysize - _Roff < _Count)
				_Count = _Right._Mysize - _Roff;	// trim _Count to size
			return (compare(_Off, _N0, _Right._Myptr() + _Roff, _Count));
		}

		int compare(const _Elem *_Ptr) const
		{	// compare [0, _Mysize) with [_Ptr, <null>)
			return (compare(0, _Mysize, _Ptr, _Traits::length(_Ptr)));
		}

		int compare(size_type _Off, size_type _N0, const _Elem *_Ptr) const
		{	// compare [_Off, _Off + _N0) with [_Ptr, <null>)
			return (compare(_Off, _N0, _Ptr, _Traits::length(_Ptr)));
		}

		int compare(size_type _Off,
			size_type _N0, const _Elem *_Ptr, size_type _Count) const
		{	// compare [_Off, _Off + _N0) with [_Ptr, _Ptr + _Count)
			if (_Mysize < _Off)
				_EL_Xran();	// _Off off end
			if (_Mysize - _Off < _N0)
				_N0 = _Mysize - _Off;	// trim _N0 to size

			size_type _Ans = _N0 == 0 ? 0
				: _Traits::compare(_Myptr() + _Off, _Ptr,
				_N0 < _Count ? _N0 : _Count);
			return (_Ans != 0 ? (int)_Ans : _N0 < _Count ? -1
				: _N0 == _Count ? 0 : +1);
		}

		enum
		{	// length of internal buffer, [1, 16]
			_BUF_SIZE = 16 / sizeof (_Elem) < 1 ? 1
			: 16 / sizeof(_Elem)};

	protected:
		enum
		{	// roundup mask for allocated buffers, [0, 15]
			_ALLOC_MASK = sizeof (_Elem) <= 1 ? 15
			: sizeof (_Elem) <= 2 ? 7
			: sizeof (_Elem) <= 4 ? 3
			: sizeof (_Elem) <= 8 ? 1 : 0};

			void _Copy(size_type _Newsize, size_type _Oldlen)
			{	// copy _Oldlen elements to newly allocated buffer
				size_type _Newres = _Newsize | _ALLOC_MASK;
				if (max_size() < _Newres)
					_Newres = _Newsize;	// undo roundup if too big
				else if (_Newres / 3 < _Myres / 2
					&& _Myres <= max_size() - _Myres / 2)
					_Newres = _Myres + _Myres / 2;	// grow exponentially if possible
				_Elem *_Ptr;

				_TRY_BEGIN
					_Ptr = _Alval.allocate(_Newres + 1);
				_CATCH_ALL
					_Newres = _Newsize;	// allocation failed, undo roundup and retry
				_TRY_BEGIN
					_Ptr = _Alval.allocate(_Newres + 1);
				_CATCH_ALL
					_Tidy(true);	// failed again, discard storage and reraise
				_RERAISE;
				_CATCH_END
				_CATCH_END

					if (0 < _Oldlen)
						_Traits::copy(_Ptr, _Myptr(), _Oldlen);	// copy existing elements
				_Tidy(true);
				_Bx._Ptr = _Ptr;
				_Myres = _Newres;
				_Eos(_Oldlen);
			}

			void _Eos(size_type _Newsize)
			{	// set new length and null terminator
				_Traits::assign(_Myptr()[_Mysize = _Newsize], _Elem());
			}

			bool _Grow(size_type _Newsize,
				bool _Trim = false)
			{	// ensure buffer is big enough, trim to size if _Trim is true
				if (max_size() < _Newsize)
					_EL_Xlen();	// result too long
				if (_Myres < _Newsize)
					_Copy(_Newsize, _Mysize);	// reallocate to grow
				else if (_Trim && _Newsize < _BUF_SIZE)
					_Tidy(true,	// copy and deallocate if trimming to small string
					_Newsize < _Mysize ? _Newsize : _Mysize);
				else if (_Newsize == 0)
					_Eos(0);	// new size is zero, just null terminate
				return (0 < _Newsize);	// return true only if more work to do
			}

			bool _Inside(const _Elem *_Ptr)
			{	// test if _Ptr points inside string
				if (_Ptr < _Myptr() || _Myptr() + _Mysize <= _Ptr)
					return (false);	// don't ask
				else
					return (true);
			}

			void _Tidy(bool _Built = false,
				size_type _Newsize = 0)
			{	// initialize buffer, deallocating any storage
				if (!_Built)
					;
				else if (_BUF_SIZE <= _Myres)
				{	// copy any leftovers to small buffer and deallocate
					_Elem *_Ptr = _Bx._Ptr;
					if (0 < _Newsize)
						_Traits::copy(_Bx._Buf, _Ptr, _Newsize);
					_Alval.deallocate(_Ptr, _Myres + 1);
				}
				_Myres = _BUF_SIZE - 1;
				_Eos(_Newsize);
			}

			union _Bxty
			{	// storage for small buffer or pointer to larger one
				_Elem _Buf[_BUF_SIZE];
				_Elem *_Ptr;
			} _Bx;

	public:
		_Elem *_Myptr()
		{	// determine current pointer to buffer for mutable string
			return (_BUF_SIZE <= _Myres ? _Bx._Ptr : _Bx._Buf);
		}

		const _Elem *_Myptr() const
		{	// determine current pointer to buffer for nonmutable string
			return (_BUF_SIZE <= _Myres ? _Bx._Ptr : _Bx._Buf);
		}

		size_type _Mysize;	// current length of string
		size_type _Myres;	// current storage reserved for string
	};

	inline _string __cdecl operator+(
		const _string& _Left,
		const _string& _Right)
	{	// return string + string
		return (_string(_Left) += _Right);
	}

	inline _string __cdecl operator+(
		const char *_Left,
		const _string& _Right)
	{	// return NTCS + string
		return (_string(_Left) += _Right);
	}

	inline _string __cdecl operator+(
		const char _Left,
		const _string& _Right)
	{	// return character + string
		return (_string(1, _Left) += _Right);
	}

	inline _string __cdecl operator+(
		const _string& _Left,
		const char *_Right)
	{	// return string + NTCS
		return (_string(_Left) += _Right);
	}

	inline _string __cdecl operator+(
		const _string& _Left,
		const char _Right)
	{	// return string + character
		return (_string(_Left) += _Right);
	}

	inline bool __cdecl operator==(
		const _string& _Left,
		const _string& _Right)
	{	// test for string equality
		return (_Left.compare(_Right) == 0);
	}

	inline bool __cdecl operator==(
		const char * _Left,
		const _string& _Right)
	{	// test for NTCS vs. string equality
		return (_Right.compare(_Left) == 0);
	}

	inline bool __cdecl operator==(
		const _string& _Left,
		const char *_Right)
	{	// test for string vs. NTCS equality
		return (_Left.compare(_Right) == 0);
	}

	inline bool __cdecl operator!=(
		const _string& _Left,
		const _string& _Right)
	{	// test for string inequality
		return (!(_Left == _Right));
	}

	inline bool __cdecl operator!=(
		const char *_Left,
		const _string& _Right)
	{	// test for NTCS vs. string inequality
		return (!(_Left == _Right));
	}

	inline bool __cdecl operator!=(
		const _string& _Left,
		const char *_Right)
	{	// test for string vs. NTCS inequality
		return (!(_Left == _Right));
	}

	inline bool __cdecl operator<(
		const _string& _Left,
		const _string& _Right)
	{	// test if string < string
		return (_Left.compare(_Right) < 0);
	}

	inline bool __cdecl operator<(
		const char * _Left,
		const _string& _Right)
	{	// test if NTCS < string
		return (_Right.compare(_Left) > 0);
	}

	inline bool __cdecl operator<(
		const _string& _Left,
		const char *_Right)
	{	// test if string < NTCS
		return (_Left.compare(_Right) < 0);
	}

	inline bool __cdecl operator>(
		const _string& _Left,
		const _string& _Right)
	{	// test if string > string
		return (_Right < _Left);
	}

	inline bool __cdecl operator>(
		const char * _Left,
		const _string& _Right)
	{	// test if NTCS > string
		return (_Right < _Left);
	}

	inline bool __cdecl operator>(
		const _string& _Left,
		const char *_Right)
	{	// test if string > NTCS
		return (_Right < _Left);
	}

	inline bool __cdecl operator<=(
		const _string& _Left,
		const _string& _Right)
	{	// test if string <= string
		return (!(_Right < _Left));
	}

	inline bool __cdecl operator<=(
		const char * _Left,
		const _string& _Right)
	{	// test if NTCS <= string
		return (!(_Right < _Left));
	}

	inline bool __cdecl operator<=(
		const _string& _Left,
		const char *_Right)
	{	// test if string <= NTCS
		return (!(_Right < _Left));
	}

	inline bool __cdecl operator>=(
		const _string& _Left,
		const _string& _Right)
	{	// test if string >= string
		return (!(_Left < _Right));
	}
	
	inline bool __cdecl operator>=(
		const char * _Left,
		const _string& _Right)
	{	// test if NTCS >= string
		return (!(_Left < _Right));
	}

	inline bool __cdecl operator>=(
		const _string& _Left,
		const char *_Right)
	{	// test if string >= NTCS
		return (!(_Left < _Right));
	}


#pragma warning(default: 4189)
#pragma warning(pop)
#pragma pack(pop)

} //namespace STLLight
