#pragma once

#include "common_Light.h"

namespace STLLight
{
	template<class _Ty, class _Alloc>
	class _Vector_val
	{
	protected:
		_Vector_val(_Alloc _Al = _Alloc())
			: _Alval(_Al)
		{}
		typedef typename _Alloc::template rebind<_Ty>::other _Alty;
		_Alty _Alval;
	};


	template<class _Ty, class _Ax = _EL_allocator<_Ty> >
	class vector
		: public _Vector_val<_Ty, _Ax>
	{
	public:
		typedef vector<_Ty, _Ax>					_Myt;
		typedef _Vector_val<_Ty, _Ax>				_Mybase;
		typedef typename _Mybase::_Alty				_Alloc;
		typedef _Alloc								allocator_type;
		typedef typename _Alloc::size_type			size_type;
		typedef typename _Alloc::difference_type	_Dift;
		typedef _Dift								difference_type;
		typedef typename _Alloc::pointer			_Tptr;
		typedef typename _Alloc::const_pointer		_Ctptr;
		typedef _Tptr								pointer;
		typedef _Ctptr								const_pointer;
		typedef typename _Alloc::reference			_Reft;
		typedef _Reft								reference;
		typedef typename _Alloc::const_reference	const_reference;
		typedef typename _Alloc::value_type			value_type;

		typedef _Tptr								iterator;
		typedef _Ctptr								const_iterator;

		vector()
			: _Mybase()
		{	// construct empty vector
			_Buy(0);
		}

		explicit vector(const _Alloc& _Al)
			: _Mybase(_Al)
		{	// construct empty vector with allocator
			_Buy(0);
		}

		explicit vector(size_type _Count)
			: _Mybase()
		{	// construct from _Count * _Ty()
			_Construct_n(_Count, _Ty());
		}

		vector(size_type _Count, const _Ty& _Val)
			: _Mybase()
		{	// construct from _Count * _Val
			_Construct_n(_Count, _Val);
		}

		vector(size_type _Count, const _Ty& _Val, const _Alloc& _Al)
			: _Mybase(_Al)
		{	// construct from _Count * _Val, with allocator
			_Construct_n(_Count, _Val);
		}

		vector(const _Myt& _Right)
			: _Mybase(_Right._Alval)
		{	// construct by copying _Right
			if (_Buy(_Right.size()))
			_TRY_BEGIN
				_Mylast = _Ucopy(_Right.begin(), _Right.end(), _Myfirst);
			_CATCH_ALL
				_Tidy();
			_RERAISE;
			_CATCH_END
		}

		template<class _Iter>
		vector(_Iter _First, _Iter _Last)
			: _Mybase()
		{	// construct from [_First, _Last)
			_Construct(_First, _Last, _Iter_cat(_First));
		}

		template<class _Iter>
		vector(_Iter _First, _Iter _Last, const _Alloc& _Al)
			: _Mybase(_Al)
		{	// construct from [_First, _Last), with allocator
			_Construct(_First, _Last, _Iter_cat(_First));
		}

		template<class _Iter>
		void _Construct(_Iter _First, _Iter _Last)
		{	// initialize with [_First, _Last), input iterators
			_Buy(0);
			_TRY_BEGIN
				insert(begin(), _First, _Last);
			_CATCH_ALL
				_Tidy();
			_RERAISE;
			_CATCH_END
		}

		void _Construct_n(size_type _Count, const _Ty& _Val)
		{	// construct from _Count * _Val
			if (_Buy(_Count))
			{	// nonzero, fill it
				_TRY_BEGIN
					_Mylast = _Ufill(_Myfirst, _Count, _Val);
				_CATCH_ALL
					_Tidy();
				_RERAISE;
				_CATCH_END
			}
		}

		~vector()
		{	// destroy the object
			_Tidy();
		}

		_Myt& operator=(const _Myt& _Right)
		{	// assign _Right
			if (this != &_Right)
			{	// worth doing

				if (_Right.size() == 0)
					clear();	// new sequence empty, erase existing sequence
				else if (_Right.size() <= size())
				{	// enough elements, copy new and destroy old
					pointer _Ptr = _STDEXT unchecked_copy(_Right._Myfirst, _Right._Mylast, _Myfirst);	// copy new
					_Destroy(_Ptr, _Mylast);	// destroy old
					_Mylast = _Myfirst + _Right.size();
				}
				else if (_Right.size() <= capacity())
				{	// enough room, copy and construct new
					pointer _Ptr = _Right._Myfirst + size();
					_STDEXT unchecked_copy(_Right._Myfirst, _Ptr, _Myfirst);
					_Mylast = _Ucopy(_Ptr, _Right._Mylast, _Mylast);
				}
				else
				{	// not enough room, allocate new array and construct new
					if (_Myfirst != 0)
					{	// discard old array
						_Destroy(_Myfirst, _Mylast);
						this->_Alval.deallocate(_Myfirst, _Myend - _Myfirst);
					}
					if (_Buy(_Right.size()))
						_Mylast = _Ucopy(_Right._Myfirst, _Right._Mylast,_Myfirst);
				}
			}
			return (*this);
		}

		void reserve(size_type _Count)
		{	// determine new minimum length of allocated storage
			if (max_size() < _Count)
				_Xlen();	// result too long
			else if (capacity() < _Count)
			{	// not enough room, reallocate
				pointer _Ptr = this->_Alval.allocate(_Count);

				_TRY_BEGIN
					_Umove(begin(), end(), _Ptr);
				_CATCH_ALL
					this->_Alval.deallocate(_Ptr, _Count);
				_RERAISE;
				_CATCH_END

				size_type _Size = size();

				if (_Myfirst != 0)
				{	// destroy and deallocate old array
					_Destroy(_Myfirst, _Mylast);
					this->_Alval.deallocate(_Myfirst, _Myend - _Myfirst);
				}

				_Myend = _Ptr + _Count;
				_Mylast = _Ptr + _Size;
				_Myfirst = _Ptr;
			}
		}

		size_type capacity() const
		{	// return current length of allocated storage
			return (_Myfirst == 0 ? 0 : _Myend - _Myfirst);
		}

		iterator begin()
		{	// return iterator for beginning of mutable sequence
			return (iterator(_Myfirst));
		}

		const_iterator begin() const
		{	// return iterator for beginning of nonmutable sequence
			return (const_iterator(_Myfirst));
		}

		iterator end()
		{	// return iterator for end of mutable sequence
			return (iterator(_Mylast));
		}

		const_iterator end() const
		{	// return iterator for end of nonmutable sequence
			return (const_iterator(_Mylast));
		}

		void resize(size_type _Newsize)
		{	// determine new length, padding with _Ty() elements as needed
			resize(_Newsize, _Ty());
		}

		void resize(size_type _Newsize, _Ty _Val)
		{	// determine new length, padding with _Val elements as needed
			if (size() < _Newsize)
				_Insert_n(end(), _Newsize - size(), _Val);
			else if (_Newsize < size())
				erase(begin() + _Newsize, end());
		}

		size_type size() const
		{	// return length of sequence
			return (_Myfirst == 0 ? 0 : _Mylast - _Myfirst);
		}

		size_type max_size() const
		{	// return maximum possible length of sequence
			return (this->_Alval.max_size());
		}

		bool empty() const
		{	// test if sequence is empty
			return (size() == 0);
		}

		_Alloc get_allocator() const
		{	// return allocator object for values
			return (this->_Alval);
		}

		const_reference at(size_type _Pos) const
		{	// subscript nonmutable sequence with checking
			if (size() <= _Pos)
				_Xran();
			return (*(begin() + _Pos));
		}

		reference at(size_type _Pos)
		{	// subscript mutable sequence with checking
			if (size() <= _Pos)
				_Xran();
			return (*(begin() + _Pos));
		}

		const_reference operator[](size_type _Pos) const
		{
			return (*(_Myfirst + _Pos));
		}

		reference operator[](size_type _Pos)
		{
			return (*(_Myfirst + _Pos));
		}

		reference front()
		{	// return first element of mutable sequence
			return (*begin());
		}

		const_reference front() const
		{	// return first element of nonmutable sequence
			return (*begin());
		}

		reference back()
		{	// return last element of mutable sequence
			return (*(end() - 1));
		}

		const_reference back() const
		{	// return last element of nonmutable sequence
			return (*(end() - 1));
		}

		void push_back(const _Ty& _Val)
		{	// insert element at end
			if (size() < capacity())
				_Mylast = _Ufill(_Mylast, 1, _Val);
			else
				insert(end(), _Val);
		}

		void pop_back()
		{
			if (!empty())
			{
				_Destroy(_Mylast - 1, _Mylast);
				--_Mylast;
			}
		}

		template<class _Iter>
		void assign(_Iter _First, _Iter _Last)
		{	// assign [_First, _Last)
			_Assign(_First, _Last);
		}

		template<class _Iter>
		void _Assign(_Iter _First, _Iter _Last)
		{	// assign [_First, _Last), input iterators
			erase(begin(), end());
			insert(begin(), _First, _Last);
		}

		void assign(size_type _Count, const _Ty& _Val)
		{	// assign _Count * _Val
			_Assign_n(_Count, _Val);
		}

		iterator insert(iterator _Where, const _Ty& _Val)
		{	// insert _Val at _Where
			size_type _Off = size() == 0 ? 0 : _Where - begin();
			_Insert_n(_Where, (size_type)1, _Val);
			return (begin() + _Off);
		}

		void insert(iterator _Where, size_type _Count, const _Ty& _Val)
		{	// insert _Count * _Val at _Where
			_Insert_n(_Where, _Count, _Val);
		}

		template<class _Iter>
		void insert(iterator _Where, _Iter _First, _Iter _Last)
		{	// insert [_First, _Last) at _Where
			_Insert(_Where, _First, _Last);
		}

		template<class _Iter>
		void _Insert(iterator _Where, _Iter _First, _Iter _Last)
		{
			size_type _Count = 0;
			_Distance(_First, _Last, _Count);
			size_type _Capacity = capacity();

			if (_Count == 0)
				;
			else if (max_size() - size() < _Count)
				_Xlen();	// result too long
			else if (_Capacity < size() + _Count)
			{	// not enough room, reallocate
				_Capacity = max_size() - _Capacity / 2 < _Capacity
					? 0 : _Capacity + _Capacity / 2;	// try to grow by 50%
				if (_Capacity < size() + _Count)
					_Capacity = size() + _Count;
				pointer _Newvec = this->_Alval.allocate(_Capacity);
				pointer _Ptr = _Newvec;

				_TRY_BEGIN
					_Ptr = _Umove(_Myfirst, (_Where),_Newvec);	// copy prefix
					_Ptr = _Ucopy(_First, _Last, _Ptr);	// add new stuff
					_Umove((_Where), _Mylast, _Ptr);	// copy suffix
				_CATCH_ALL
					_Destroy(_Newvec, _Ptr);
					this->_Alval.deallocate(_Newvec, _Capacity);
				_RERAISE;
				_CATCH_END

				_Count += size();
				if (_Myfirst != 0)
				{	// destroy and deallocate old array
					_Destroy(_Myfirst, _Mylast);
					this->_Alval.deallocate(_Myfirst, _Myend - _Myfirst);
				}

				_Myend = _Newvec + _Capacity;
				_Mylast = _Newvec + _Count;
				_Myfirst = _Newvec;
			}
			else if ((size_type)(end() - _Where) < _Count)
			{	// new stuff spills off end
				_Umove((_Where), _Mylast,(_Where) + _Count);
				_Iter _Mid = _First;
				_Advance(_Mid, end() - _Where);

				_TRY_BEGIN
					_Ucopy(_Mid, _Last, _Mylast);	// insert new stuff off end
				_CATCH_ALL
					_Destroy((_Where) + _Count, _Mylast + _Count);
				_RERAISE;
				_CATCH_END

				_Mylast += _Count;

				_STDEXT unchecked_copy(_First, _Mid, (_Where));
			}
			else
			{	// new stuff can all be assigned
				pointer _Oldend = _Mylast;
				_Mylast = _Umove(_Oldend - _Count, _Oldend, _Mylast);	// copy suffix
				_STDEXT _Unchecked_move_backward((_Where), _Oldend - _Count, _Oldend);	// copy hole
				_STDEXT unchecked_copy(_First, _Last, (_Where));	// insert into hole
			}
		}

		iterator erase(iterator _Where)
		{	// erase element at where
			_STDEXT unchecked_copy((_Where) + 1, _Mylast, (_Where));
			_Destroy(_Mylast - 1, _Mylast);
			--_Mylast;
			return (_Where);
		}

		iterator erase(iterator _First, iterator _Last)
		{	// erase [_First, _Last)
			if (_First != _Last)
			{	// worth doing, copy down over hole
				pointer _Ptr = _STDEXT unchecked_copy((_Last), _Mylast, (_First));
				_Destroy(_Ptr, _Mylast);
				_Mylast = _Ptr;
			}
			return (_First);
		}

		void clear()
		{	// erase all
			erase(begin(), end());
		}

		void swap(_Myt& _Right)
		{
			if (this->_Alval == _Right._Alval)
			{
				_STD swap(_Myfirst, _Right._Myfirst);
				_STD swap(_Mylast, _Right._Mylast);
				_STD swap(_Myend, _Right._Myend);
			}
			else
			{
				_Myt _Ts = *this; *this = _Right, _Right = _Ts;
			}
		}

	protected:
		void _Assign_n(size_type _Count, const _Ty& _Val)
		{	// assign _Count * _Val
			_Ty _Tmp = _Val;	// in case _Val is in sequence
			erase(begin(), end());
			insert(begin(), _Count, _Tmp);
		}

		bool _Buy(size_type _Capacity)
		{	// allocate array with _Capacity elements
			_Myfirst = 0, _Mylast = 0, _Myend = 0;
			if (_Capacity == 0)
				return (false);
			else if (max_size() < _Capacity)
				_Xlen();	// result too long
			else
			{	// nonempty array, allocate storage
				_Myfirst = this->_Alval.allocate(_Capacity);
				_Mylast = _Myfirst;
				_Myend = _Myfirst + _Capacity;
			}
			return (true);
		}

		void _Destroy(pointer _First, pointer _Last)
		{	// destroy [_First, _Last) using allocator
			_STD _Destroy_range(_First, _Last, this->_Alval);
		}

		void _Tidy()
		{	// free all storage
			if (_Myfirst != 0)
			{	// something to free, destroy and deallocate it
				_Destroy(_Myfirst, _Mylast);
				this->_Alval.deallocate(_Myfirst, _Myend - _Myfirst);
			}
			_Myfirst = 0, _Mylast = 0, _Myend = 0;
		}

		template<class _Iter>
		pointer _Ucopy(_Iter _First, _Iter _Last, pointer _Ptr)
		{	// copy initializing [_First, _Last), using allocator
			return (_STDEXT unchecked_uninitialized_copy(_First, _Last, _Ptr, this->_Alval));
		}

		template<class _Iter>
		pointer _Umove(_Iter _First, _Iter _Last, pointer _Ptr)
		{	// move initializing [_First, _Last), using allocator
			return (_STDEXT _Unchecked_uninitialized_move(_First, _Last, _Ptr, this->_Alval));
		}

		void _Insert_n(iterator _Where, size_type _Count, const _Ty& _Val)
		{	// insert _Count * _Val at _Where

			_Ty _Tmp = _Val;	// in case _Val is in sequence
			size_type _Capacity = capacity();

			if (_Count == 0)
				;
			else if (max_size() - size() < _Count)
				_Xlen();	// result too long
			else if (_Capacity < size() + _Count)
			{	// not enough room, reallocate
				_Capacity = max_size() - _Capacity / 2 < _Capacity
					? 0 : _Capacity + _Capacity / 2;	// try to grow by 50%
				if (_Capacity < size() + _Count)
					_Capacity = size() + _Count;
				pointer _Newvec = this->_Alval.allocate(_Capacity);
				pointer _Ptr = _Newvec;

				_TRY_BEGIN
					_Ptr = _Umove(_Myfirst, (_Where), _Newvec);	// copy prefix
					_Ptr = _Ufill(_Ptr, _Count, _Tmp);	// add new stuff
					_Umove((_Where), _Mylast, _Ptr);	// copy suffix
				_CATCH_ALL
					_Destroy(_Newvec, _Ptr);
					this->_Alval.deallocate(_Newvec, _Capacity);
				_RERAISE;
				_CATCH_END

				_Count += size();
				if (_Myfirst != 0)
				{	// destroy and deallocate old array
					_Destroy(_Myfirst, _Mylast);
					this->_Alval.deallocate(_Myfirst, _Myend - _Myfirst);
				}

				_Myend = _Newvec + _Capacity;
				_Mylast = _Newvec + _Count;
				_Myfirst = _Newvec;
			}
			else if ((size_type)(_Mylast - (_Where)) < _Count)
			{	// new stuff spills off end
				_Umove((_Where), _Mylast, (_Where) + _Count);	// copy suffix

				_TRY_BEGIN
					_Ufill(_Mylast, _Count - (_Mylast - (_Where)), _Tmp);	// insert new stuff off end
				_CATCH_ALL
					_Destroy((_Where) + _Count, _Mylast + _Count);
				_RERAISE;
				_CATCH_END

				_Mylast += _Count;

				_STD _Fill((_Where), _Mylast - _Count, _Tmp);	// insert up to old end
			}
			else
			{	// new stuff can all be assigned
				pointer _Oldend = _Mylast;
				_Mylast = _Umove(_Oldend - _Count, _Oldend, _Mylast);	// copy suffix

				_STDEXT _Unchecked_move_backward((_Where), _Oldend - _Count, _Oldend);	// copy hole

				_STD _Fill((_Where), (_Where) + _Count, _Tmp);	// insert into hole
			}
		}

		pointer _Ufill(pointer _Ptr, size_type _Count, const _Ty &_Val)
		{	// copy initializing _Count * _Val, using allocator
			_STDEXT unchecked_uninitialized_fill_n(_Ptr, _Count, _Val, this->_Alval);
			return (_Ptr + _Count);
		}

		static void _Xlen()
		{
			_EL_Xlen();
		}

		static void _Xran()
		{
			_EL_Xran();
		}

		static void _Xinvarg()
		{
			_EL_Xran();
		}

		pointer _Myfirst;	// pointer to beginning of array
		pointer _Mylast;	// pointer to current end of sequence
		pointer _Myend;	// pointer to end of array
	};

	template<class _Ty,
	class _Alloc> inline
		bool operator==(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test for vector equality
		return (_Left.size() == _Right.size()
			&& _STD equal(_Left.begin(), _Left.end(), _Right.begin()));
	}

	template<class _Ty,
	class _Alloc> inline
		bool operator!=(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test for vector inequality
		return (!(_Left == _Right));
	}

	template<class _Ty,
	class _Alloc> inline
		bool operator<(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test if _Left < _Right for vectors
		return (_STD lexicographical_compare(_Left.begin(), _Left.end(),
			_Right.begin(), _Right.end()));
	}

	template<class _Ty,
	class _Alloc> inline
		bool operator>(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test if _Left > _Right for vectors
		return (_Right < _Left);
	}

	template<class _Ty,
	class _Alloc> inline
		bool operator<=(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test if _Left <= _Right for vectors
		return (!(_Right < _Left));
	}

	template<class _Ty,
	class _Alloc> inline
		bool operator>=(const vector<_Ty, _Alloc>& _Left,
		const vector<_Ty, _Alloc>& _Right)
	{	// test if _Left >= _Right for vectors
		return (!(_Left < _Right));
	}

	template<class _Ty,
	class _Alloc> inline
		void swap(vector<_Ty, _Alloc>& _Left, vector<_Ty, _Alloc>& _Right)
	{	// swap _Left and _Right vectors
		_Left.swap(_Right);
	}

};//namespace STLLight
